#!/usr/bin/env python3
# -*- coding: utf-8 -*-

#   Copyright 2010-2013 Tuukka Turto
#
#   This file is part of pyherc.
#
#   pyherc is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#
#   pyherc is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with pyherc.  If not, see <http://www.gnu.org/licenses/>.

"""
Module for start game related functionality
"""
class StartGameScreen():
    """
    Screen to start game

    .. versionadded:: 0.9
    """
    def __init__(self, generator, application, screen):
        """
        Default constructor
        """
        super(StartGameScreen, self).__init__()

        self.generator = generator
        self.class_names = self.generator.configuration.keys()
        self.screen = screen


    def show(self):
        """
        Show this screen
        """
        self.draw_screen()

        selection = None

        while selection == None:
            selection = self.screen.getch()

            try:
                selection = int(chr(selection))
            except ValueError:
                selection = None

            if selection < 0 or selection > len(self.class_names) - 1:
                selection = None

        for index, class_name in enumerate(self.class_names):
            if index == selection:
                return self.generator.generate_creature(class_name)

    def draw_screen(self):
        """
        Draw screen
        """
        self.screen.clear()
        for index, class_name in enumerate(self.class_names):
            self.screen.addstr(5 + index, 20,
                               '{0}. {1}'.format(index, class_name))
        self.screen.refresh()



"""{-
Copyright (c) 2009 Pauli Henrikki Rikula

Permission is hereby granted, free of charge, to any person
obtaining a copy of this software and associated documentation
files (the "Software"), to deal in the Software without
restriction, including without limitation the rights to use,
copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the
Software is furnished to do so, subject to the following
conditions:

The above copyright notice and this permission notice shall be
included in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
OTHER DEALINGS IN THE SOFTWARE.
-}"""


from pyherc.ai import pyheapq

from pyherc.ai.heapset import HeapSet
from copy import copy


def reconstruct_path(came_from, current_node):
    """
    came_from : dictuonary of nodes and nodes before them in the shortest found path
    current_node : the last node of the path
    """
    if current_node in came_from:
        p = reconstruct_path(came_from, came_from[current_node])
        return p + [current_node]
    else:
        return []


class HeapItem:
    def __init__(self, y, goal, a_map, g_score):

        self.node = y

        """ g_score = Distance from start along optimal path."""
        self.g_score = g_score

        """h_score the heuristic estimates of the distances to goal"""
        self.h_score = a_map.heuristic_estimate_of_distance(y, goal)

        """f_score Estimated total distance from start to goal through y."""
        self.f_score = self.h_score + self.g_score

    def as_tuple(self):
        return (self.f_score, self.g_score, self.h_score, self.node)

    def __hash__(self):
        return self.as_tuple().__hash__()

    def __repr__(self):
        return str(self.as_tuple())

    def type_check(self, other):
        return type(self) == type(other)
    def __lt__(self, other):
        return self.type_check(other) and self.as_tuple().__lt__(other.as_tuple())
    def __le__(self, other):
        return self.type_check(other) and self.as_tuple().__le__(other.as_tuple())
    def __eq__(self, other):
        return self.type_check(other) and self.as_tuple().__eq__(other.as_tuple())
    def __ne__(self, other):
        return self.type_check(other) and self.as_tuple().__ne__(other.as_tuple())
    def __gt__(self, other):
        return self.type_check(other) and self.as_tuple().__gt__(other.as_tuple())
    def __ge__(self, other):
        return self.type_check(other) and self.as_tuple().__ge__(other.as_tuple())




def a_star(start, goal, a_map):
    """
    start = the start node in a_map
    goal = the goal node in a_map
    a_map = a object that should inherit the Map class

    returns a tuple (path, connections, uptated) where:
      path is the optimal path (as a list of points) from the start to the goal. empty if not found,

      connections and updated are for debugging (remove them from code if too slow..,):
        connections is the came_from dictionary and
        uptated is the set of the connections, which were uptated from the heap

    """

    """The set of nodes already evaluated."""
    closedset = set([])


    firstItem = HeapItem(start, goal, a_map, 0.0)

    lastItem = None

    """
    openDict is the set of tentative nodes to be evaluated
    containing just the initial node

    scoreHeap is used as priority queue for next steps.
    """

    scoreHeap = HeapSet([firstItem])
    openDict = {start:firstItem}


    """ the second last node in the shortest path from start node"""
    came_from = {}

    """this is the set of points which were uptated when they were in the heap
    this is used only to debug the algorithm. remove if slows too much"""
    updateset = set([])

    while any(scoreHeap): # is not empty
        """
        the node in openset having
        the lowest (f_score,g_score,h_score, position) value (f_score means the most ...)
        """
        x  = pyheapq.heappop(scoreHeap)

        if x.node == goal:
            if lastItem == None or lastItem.g_score > x.g_score:
                lastItem = x

            if not any(scoreHeap) or scoreHeap[0].f_score > lastItem.g_score:
                return [start] + reconstruct_path(came_from, goal), came_from, updateset

        del openDict[x.node]
        closedset.add(x.node)

        neighbornodes =  [
            (x.g_score + a_map.dist_between(x.node, node_y),node_y )
            for node_y in a_map.neighbor_nodes(x.node)
            ]
        #better sort here than update the heap ..
        neighbornodes.sort()


        for tentative_g_score, node_y in neighbornodes:

            if node_y in closedset:
                continue


            oldy = openDict.get(node_y, None)
            y = copy(oldy)

            y = HeapItem(node_y, goal, a_map, tentative_g_score)

            if oldy == None:
                openDict[node_y] = y
                came_from[node_y] = x.node

                pyheapq.heappush(scoreHeap, y)

            elif tentative_g_score < oldy.g_score:
                updateset.add( (node_y, came_from[node_y]) )

                openDict[node_y] = y
                came_from[node_y] = x.node

                pyheapq.updateheapvalue(scoreHeap, scoreHeap.index(oldy), y)

    if lastItem != None:
        return [start] + reconstruct_path(came_from,goal), came_from, updateset
    else:
        return [], came_from, updateset



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

#   Copyright 2010-2013 Tuukka Turto
#
#   This file is part of pyherc.
#
#   pyherc is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#
#   pyherc is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with pyherc.  If not, see <http://www.gnu.org/licenses/>.

"""
Module for spell factory
"""
from pyherc.aspects import logged

from pyherc.data.magic import Spell
from pyherc.data.effects import EffectHandle

class SpellGenerator():
    """
    Factory for creating spells
    
    .. versionadded:: 0.9
    """
    @logged
    def __init__(self):
        """
        Default constructor
        """
        self.spell_list = {}

        self.__init_spells()

    @logged
    def __init_spells(self):
        """
        Temporary implementation for spell loading
        """

        healing_spell = SpellSpecification([
                              EffectHandle(trigger = 'on spell hit',
                                           effect = 'heal medium wounds',
                                           parameters = None,
                                           charges = 1)],
                              targeting_caster)

        magic_missile = SpellSpecification([
                            EffectHandle(trigger = 'on spell hit',
                                         effect = 'cause wound',
                                         parameters = None,
                                         charges = 1)],
                            targeting_single_target)

        self.spell_list['healing wind'] = healing_spell
        self.spell_list['magic missile'] = magic_missile
    
    @logged
    def create_spell(self, spell_name, targets):
        """
        Create a spell
        
        :param spell_name: name of the spell
        :type spell_name: string
        :param target: target of the spell
        :type target: Character
        :returns: ready to use spell
        :rtype: Spell
        """
        new_spell = Spell()
        new_spell.targets.extend(targets)

        spec = self.spell_list[spell_name]
        handles = spec.effect_handles

        for effect_handle in handles:
            new_spell.add_effect_handle(effect_handle)

        return new_spell

class SpellSpecification():
    """
    Class to specify spell configuration

    .. versionadded:: 0.9
    """
    def __init__(self, effect_handles, targeter):
        self.effect_handles = effect_handles
        self.targeter = targeter
    
def targeting_caster(parameters):
    """
    Function to target the caster

    .. versionadded:: 0.9
    """
    return [parameters.caster]

def targeting_single_target(parameters):
    """
    Function to target a single character in direction

    .. versionadded:: 0.9
    """
    return None




#!/usr/bin/env python3
# -*- coding: utf-8 -*-

#   Copyright 2010-2013 Tuukka Turto
#
#   This file is part of pyherc.
#
#   pyherc is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#
#   pyherc is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with pyherc.  If not, see <http://www.gnu.org/licenses/>.

"""
Drinking and eating related factories are defined here
"""
from pyherc.rules.consume.action import DrinkAction
from pyherc.rules.factory import SubActionFactory

class DrinkFactory(SubActionFactory):
    """
    Factory for creating drink actions
    """
    def __init__(self, effect_factory, dying_rules):
        """
        Constructor for this factory
        """
        super(DrinkFactory, self).__init__(effect_factory)
        self.action_type = 'drink'
        self.dying_rules = dying_rules

    def get_action(self, parameters):
        """
        Create a drink action

        Args:
            parameters: Parameters used to control drink action creation
        """
        return DrinkAction(parameters.character,
                           parameters.item,
                           self.effect_factory,
                           self.dying_rules)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

#   Copyright 2010-2013 Tuukka Turto
#
#   This file is part of pyherc.
#
#   pyherc is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#
#   pyherc is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with pyherc.  If not, see <http://www.gnu.org/licenses/>.

"""
Module for basic decorators
"""

import logging

class Decorator():
    """
    Super class for level decorators
    """
    def __init__(self, configuration):
        """
        Default constructor

        :param configuration: configuration for decorator
        """
        super(Decorator, self).__init__()
        self.configuration = configuration
        self.logger = logging.getLogger('pyherc.generators.level.decorator.basic.Decorator')

    def __get_level_types(self):
        """
        Get types of levels this decorator supports

        :returns: level types
        :rtype: [string]
        """
        return self.configuration.level_types

    level_types = property(__get_level_types)

    def decorate_level(self, level):
        """
        Decorate level

        :param level: Level to decorate
        :type level: Level
        """
        pass

class DecoratorConfig():
    """
    Super class for decorator configuration
    """
    def __init__(self, level_types):
        """
        Default constructor

        :param level_types: level types to handle
        :type level_types: [string]
        """
        super(DecoratorConfig, self).__init__()
        self.level_types = level_types

class ReplacingDecorator(Decorator):
    """
    Simple decorator used to replace prototiles with real ones
    """
    def __init__(self, configuration):
        """
        Default constructor

        :param configuration: configuration specifying tiles to replace
        :type configuration: ReplacingDecoratorConfig
        """
        super(ReplacingDecorator, self).__init__(configuration)
        self.logger = logging.getLogger('pyherc.generators.level.decorator.basic.ReplacingDecorator')

    def decorate_level(self, level):
        """
        Decorate level

        :param level: level to decorate
        :type level: Level
        """
        ground_tiles = self.configuration.ground_config
        wall_tiles = self.configuration.wall_config

        for loc_y in range(len(level.floor[0])):
            for loc_x in range(len(level.floor)):
                proto_tile = level.floor[loc_x][loc_y]
                if proto_tile in ground_tiles:
                    level.floor[loc_x][loc_y] = ground_tiles[proto_tile]

                proto_tile = level.walls[loc_x][loc_y]
                if proto_tile in wall_tiles:
                    level.walls[loc_x][loc_y] = wall_tiles[proto_tile]


class ReplacingDecoratorConfig(DecoratorConfig):
    """
    Configuration for ReplacingDecorator
    """
    def __init__(self, level_types, ground_config, wall_config):
        """
        Default constructor

        :param level_types: types of level to handle
        :type level_types: [string]
        :param ground_config: configuration for ground
        :param wall_config: configuration for walls
        """
        super(ReplacingDecoratorConfig, self).__init__(level_types)
        self.ground_config = ground_config
        self.wall_config = wall_config

class WallBuilderDecorator(Decorator):
    """
    Decorator used to build walls

    This decorator will search all positions where there are walls next to empty
    space and replace tiles there with specified ones
    """
    def __init__(self, configuration):
        """
        Default constructor

        :param configuration: configuration
        :type configuration: WallBuilderDecoratorConfig
        """
        super(WallBuilderDecorator, self).__init__(configuration)
        self.logger = logging.getLogger('pyherc.generators.level.decorator.basic.WallBuilderDecorator')

    def decorate_level(self, level):
        """
        Decorate level

        :param level: level to decorate
        :type level: Level
        """
        for loc_y in range(1, len(level.floor[0]) - 1):
            for loc_x in range(1, len(level.floor) - 1):
                if level.walls[loc_x][loc_y] == self.configuration.empty_tile:
                    self.check_and_replace((loc_x - 1, loc_y), level)
                    self.check_and_replace((loc_x + 1, loc_y), level)
                    self.check_and_replace((loc_x, loc_y - 1), level)
                    self.check_and_replace((loc_x, loc_y + 1), level)
                    self.check_and_replace((loc_x - 1, loc_y - 1), level)
                    self.check_and_replace((loc_x - 1, loc_y + 1), level)
                    self.check_and_replace((loc_x + 1, loc_y - 1), level)
                    self.check_and_replace((loc_x + 1, loc_y + 1), level)

    def check_and_replace(self, location, level):
        """
        Check location and replace tile if it matches configuration

        :param location: location to check
        :type location: (integer, integer)
        :param level: level to use
        :type level: Level
        """
        loc_x = location[0]
        loc_y = location[1]
        proto_tile = level.walls[loc_x][loc_y]

        if proto_tile in self.configuration.wall_config:
            tile = self.configuration.wall_config[proto_tile]
            level.walls[loc_x][loc_y] = tile

class WallBuilderDecoratorConfig(DecoratorConfig):
    """
    Configuration for WallBuilderDecorator
    """
    def __init__(self, level_types, wall_config, empty_tile):
        """
        Default constructor

        :param level_types: types of level to handle
        :type level_types: [string]
        :param wall_config: configuration for walls
        :param empty_tile: tile ID to use for empty spaces
        :type empty_tile: integer
        """
        super(WallBuilderDecoratorConfig, self).__init__(level_types)
        self.wall_config = wall_config
        self.empty_tile = empty_tile

class AggregateDecorator(Decorator):
    """
    Decorator that consists of multiple decorators
    """
    def __init__(self, configuration):
        """
        Default constructor

        :param configuration: configuration
        :type configuration: AggregateDecoratorConfig
        """
        super(AggregateDecorator, self).__init__(configuration)
        self.logger = logging.getLogger('pyherc.generators.level.decorator.basic.AggregateDecorator')

    def decorate_level(self, level):
        """
        Decorate level

        :param level: level to decorate
        :type level: Level
        """
        for decorator in self.configuration.decorators:
            decorator.decorate_level(level)

class AggregateDecoratorConfig(DecoratorConfig):
    """
    Configuration for AggregateDecorator
    """
    def __init__(self, level_types, decorators):
        """
        Default constructor

        :param level_types: types of levels to handle
        :type level_types: [string]
        :decorators: decorators to group
        :type decorators: [Decorator]
        """
        super(AggregateDecoratorConfig, self).__init__(level_types)
        self.decorators = decorators



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

#   Copyright 2010-2013 Tuukka Turto
#
#   This file is part of pyherc.
#
#   pyherc is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#
#   pyherc is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with pyherc.  If not, see <http://www.gnu.org/licenses/>.

"""
Simple AI for flocking creature

Creature will try to find friends, before attacking the player character
"""

import math
from pyherc.aspects import logged

class FlockingHerbivore():
    """
    AI for flocking herbivore
    Tries to maintain close distance to other animals
    Seeks out player for combat
    """
    @logged
    def __init__(self, character):
        """
        Default constructor

        :param character: character to connect
        :type character: Character
        """
        self.character = character

    @logged
    def act(self, model, action_factory, rng):
        """
        Trigger this AI to assess the situation and act accordingly

        :param model: model where the character is located
        :type model: Model
        :param action_factory: factory to create actions
        :type action_factory: ActionFactory
        :param rng: random number generator
        :type rng: Random
        """
        shortest_distance = None
        closest_creature = None
        character = self.character
        player = model.player

        del character.short_term_memory[:]

        for creature in character.level.creatures:
            if creature != character:
                loc_x = abs(creature.location[0] - character.location[0])
                loc_y = abs(creature.location[1] - character.location[1])
                distance = math.sqrt(loc_x * loc_x + loc_y * loc_y)
                if shortest_distance != None:
                    if distance < shortest_distance:
                        shortest_distance = distance
                        closest_creature = creature
                else:
                    shortest_distance = distance
                    closest_creature = creature

        if shortest_distance != None:
            if shortest_distance <= 2:
                #seek player instead
                loc_x = abs(player.location[0] - character.location[0])
                loc_y = abs(player.location[1] - character.location[1])
                distance = math.sqrt(loc_x * loc_x + loc_y * loc_y)

                direction = self.find_direction(
                                        character.location, player.location)

                if distance > 1.42:
                    if character.is_move_legal(direction,
                                               'walk',
                                               action_factory):
                        character.move(direction,
                                       action_factory)
                    else:
                        character.tick = character.tick + 10
                else:
                    #attack
                    character.perform_attack(direction,
                                             action_factory,
                                             rng)
            else:
                #find direction
                direction = self.find_direction(character.location,
                                                    closest_creature.location)
                if character.is_move_legal(direction,
                                           'walk',
                                           action_factory):
                    character.move(direction,
                                   action_factory)
                else:
                    character.tick = character.tick + 10
        else:
            #we're all alone here
            character.tick = character.tick + 10

    @logged
    def find_direction(self, start, end):
        """
        Find direction from start to end

        :param start: start location
        :type start: (integer, integer)
        :param end: end location
        :type end: (integer, integer)

        :returns: Direction to travel
        :rtype: integer
        """
        assert(start != None)
        assert(end != None)

        direction = None
        if start[0] < end[0]:
            #right side
            if start[1] < end[1]:
                #right, below
                direction = 4
            elif start[1] > end[1]:
                #right, above
                direction = 2
            else:
                #right
                direction = 3
        elif start[0] > end[0]:
            #left side
            if start[1] < end[1]:
                #left, below
                direction = 6
            elif start[1] > end[1]:
                #left, above
                direction = 8
            else:
                #left
                direction = 7
        else:
            #up or down
            if start[1] < end[1]:
                #below
                direction = 5
            elif start[1] > end[1]:
                #above
                direction = 1

        return direction



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

#   Copyright 2010-2013 Tuukka Turto
#
#   This file is part of pyherc.
#
#   pyherc is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#
#   pyherc is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with pyherc.  If not, see <http://www.gnu.org/licenses/>.

"""
Public interface for action subsystem

Classes:
ActionFactory - Class used to contruct Action objects
ActionParameters - Class used to guide Action construction

AttackParameters - Class used to guide contruction of attack related actions
MoveParameters - Class used to guide construction of move related actions
DrinkParameters - Class used to guide drinking related actions
InventoryParameters - Class used to guide inventory related actions
SpellCastingParameteres - Class used to guide spell casting
"""

from pyherc.aspects import logged

class ActionFactory():
    """
    Object for creating actions
    """

    @logged
    def __init__(self, model, factories):
        """
        Construct ActionFactory

        Args:
            model: model to register for the factory
            factories: a single Factory or list of Factories to use
        """
        if hasattr(factories, '__iter__'):
            self.factories = factories
        else:
            self.factories = []
            self.factories.append(factories)

        self.model = model

    @logged
    def get_action(self, parameters):
        """
        Create an action

        Args:
            parameters: Parameters used to control action creation
        """
        factory = self.get_sub_factory(parameters)
        assert factory != None, 'suitable factory not configured'
        return factory.get_action(parameters)

    @logged
    def get_sub_factories(self):
        """
        Get all sub factories

        Returns:
            List of sub factories
        """
        return self.factories

    @logged
    def get_sub_factory(self, parameters):
        """
        Get sub factory to handle parameters

        Args:
            parameters: Parameters to use for searching the factory

        Returns:
            Sub factory if found, None otherwise
        """
        subs = [x for x in self.factories if x.can_handle(parameters)]

        if len(subs) == 1:
            return subs[0]
        else:
            return None

class ActionParameters():
    """
    Object for controlling action creation
    """

    @logged
    def __init__(self):
        """
        Default constructor
        """
        self.action_type = 'default'

class AttackParameters(ActionParameters):
    """
    Object for controlling attack action creation
    """

    @logged
    def __init__(self, attacker, direction, attack_type,
                 random_number_generator):
        """
        Construct AttackParameters

        Args:
            attacker: Character doing an attack
            direction: Direction to attack to
            attack_type: type of attack to perform
            random_number_generator: Random number generator to use
        """
        ActionParameters.__init__(self)

        self.action_type = 'attack'
        self.attacker = attacker
        self.direction = direction
        self.attack_type = attack_type
        self.random_number_generator = random_number_generator
        self.model = None

    def __str__(self):
        """
        Get string representation of this object
        """
        return 'attack with attack type of ' + self.attack_type

class MoveParameters(ActionParameters):
    """
    Object for controlling move action creation
    """
    @logged
    def __init__(self, character, direction, movement_mode):
        """
        Construct move parameters

        Args:
            character: Character moving
            direction: Direction of the move
            movement_mode: Mode of movement
        """
        ActionParameters.__init__(self)

        self.action_type = 'move'
        self.character = character
        self.direction = direction
        self.movement_mode = movement_mode
        self.model = None

    def __str__(self):
        """
        Get string representation of this object
        """
        return 'move with movement mode of ' + self.movement_mode

class DrinkParameters(ActionParameters):
    """
    Object for controlling drink action creation
    """
    @logged
    def __init__(self, character, item):
        """
        Construct drink parameters

        Args:
            character: Character moving
            item: Item to drink
        """
        ActionParameters.__init__(self)

        self.action_type = 'drink'
        self.character = character
        self.item = item
        self.model = None

    def __str__(self):
        """
        Get string representation of this object
        """
        return 'drink parameters'

class InventoryParameters(ActionParameters):
    """
    Class for controlling inventory action creation
    """
    @logged
    def __init__(self, character, item, sub_action):
        """
        Construct inventory parameters

        :param character: character manipulating the item
        :type character: Character
        :param item: item to manipulate
        :type item: Item
        :param sub_action: type of action to perform
        :type sub_action: string
        """
        ActionParameters.__init__(self)

        self.action_type = 'inventory'
        self.sub_action = sub_action
        self.character = character
        self.item = item

class SpellCastingParameters(ActionParameters):
    """
    Class for controlling spell casting
    
    .. versionadded:: 0.9
    """
    
    @logged
    def __init__(self, caster, direction, spell_name):
        """
        Construct spell casting parameters

        :param caster: character casting the spell
        :type caster: Character
        :param direction: direction to which the spell is cast
        :type direction: int
        :param spell_name: name of the spell
        :type spell_name: string
        """
        ActionParameters.__init__(self)

        self.action_type = 'spell casting'
        self.caster = caster
        self.direction = direction
        self.spell_name = spell_name



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

#   Copyright 2010-2013 Tuukka Turto
#
#   This file is part of pyherc.
#
#   pyherc is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#
#   pyherc is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with pyherc.  If not, see <http://www.gnu.org/licenses/>.

"""
Module defining Melee Attack related objcts

Classes:
    MeleeToHit
    MeleeDamage
"""
import random
from pyherc.aspects import logged
from pyherc.rules.attack.action import ToHit
from pyherc.rules.attack.action import Damage

class MeleeToHit(ToHit):
    """
    Class to perform to hit calculations in melee combat
    """

    @logged
    def __init__(self, attacker,  target,
                        random_number_generator = random.Random()):
        """
        Default constructor

        Args:
            attacker: Character doing the attack
            target: Character being attacked
            rng: Random number generator
        """
        self.attacker = attacker
        self.target = target
        self.rng = random_number_generator

class MeleeDamage(Damage):
    """
    Damage done in unarmed attack
    """

    @logged
    def __init__(self, damage):
        """
        Default constructor
        """
        super(MeleeDamage, self).__init__(damage)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

#   Copyright 2010-2013 Tuukka Turto
#
#   This file is part of pyherc.
#
#   pyherc is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#
#   pyherc is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with pyherc.  If not, see <http://www.gnu.org/licenses/>.

"""
Module for item related classes

Classes:
    Item
    WeaponData
"""
from pyherc.aspects import logged

class Item():
    """
    Represents item
    """
    @logged
    def __init__(self, effects_collection):
        """
        Default constructor

        :param effects_collection: collection for effects
        :type effects_collection: EffectsCollection
        """
        super(Item, self).__init__()
        #attributes
        self.name = 'prototype'
        self.description = 'prototype'
        self.appearance = ''
        self.quest_item = 0
        #location
        self.location = ()
        self.level = None
        #icon
        self.icon = None
        self.weapon_data = None
        self.armour_data = None
        self.ammunition_data = None
        self.__effects_collection = effects_collection
        self.weight = None
        self.rarity = None
        self.cost = None
        self.tags = {}
        self.__update_listeners = []

    def __str__(self):
        return self.name

    @logged
    def get_name(self, character, decorate = False):
        """
        Get name of the item
        Name can be appearance or given name

        :param character: character handling the item
        :type character: Character
        :param decorate: should name be decorated with status info, default False
        :type decorate: Boolean
        """
        assert character != None

        if self.appearance != '':
            if self.name in character.item_memory:
                name = character.item_memory[self.name]
            else:
                name = self.appearance
        else:
            name = self.name

        if decorate == True:
            if self.ammunition_data != None:
                name = name + ' ({0})'.format(self.ammunition_data.count)
            if character.inventory.weapon == self:
                name = name + ' (weapon in hand)'
            elif character.inventory.armour == self:
                name = name + ' (armour in use)'
            elif character.inventory.projectiles == self:
                name = name + ' (quivered)'

        return name

    def add_effect_handle(self, handle):
        """
        Adds an effect handle to an item

        :param handle: effect handle to add
        :type handle: EffectHandle

        .. versionadded:: 0.4
        """
        self.__effects_collection.add_effect_handle(handle)

    @logged
    def get_effect_handles(self, trigger = None):
        """
        Retrieves effects handles the item has

        :param trigger: type of effects retrieved. Default None
        :type trigger: string

        :returns: effect handles
        :rtype: [EffectHandle]

        .. versionadded:: 0.4
        """
        return self.__effects_collection.get_effect_handles(trigger)

    def __get_charges_left(self):
        """
        Amount of charges left in collection

        :returns: amount of charges
        :rtype: [integer]
        """
        return self.__effects_collection.get_charges_left()

    def __get_maximum_charges_left(self):
        """
        Return highest amount of charges left in collection

        :returns: highest charge
        :rtype: integer
        """
        return self.__effects_collection.get_maximum_charges_left()

    def __get_minimum_charges_left(self):
        """
        Return smallest amount of charges left in item

        :returns: smallest charge in collection
        :rtype: integer
        """
        return self.__effects_collection.get_minimum_charges_left()

    charges_left = property(__get_charges_left)
    maximum_charges_left = property(__get_maximum_charges_left)
    minimum_charges_left = property(__get_minimum_charges_left)

    @logged
    def get_main_type(self):
        """
        Return main type of the item

        :returns: main type
        :rtype: string
        """
        main_type = 'undefined'

        if 'weapon' in self.tags:
            main_type = 'weapon'
        elif 'armour' in self.tags:
            main_type = 'armour'
        elif 'potion' in self.tags:
            main_type = 'potion'
        elif 'ammunition' in self.tags:
            main_type = 'ammunition'
        elif 'food' in self.tags:
            main_type = 'food'

        return main_type

    @logged
    def get_tags(self):
        """
        Return tags
        """
        return self.tags

    @logged
    def register_for_updates(self, listener):
        """
        Register listener to receive updates for this entity

        :param listener: listener to add
        :type listener: Listener

        .. versionadded:: 0.5
        """
        self.__update_listeners.append(listener)

    @logged
    def remove_from_updates(self, listener):
        """
        Remove listener

        :param listener: listener to remove
        :type listener: Listener

        .. versionadded:: 0.5
        """
        self.__update_listeners.remove(listener)

    @logged
    def notify_update_listeners(self, event):
        """
        Notify all listeners registered for update of this entity

        :param event: event to relay to update listeners
        :type event: Event

        .. versionadded:: 0.5
        """
        for listener in self.__update_listeners:
            listener.receive_update(event)

    def _repr_pretty_(self, p, cycle):
        """
        Pretty print for IPython

        :param p: printer to write
        :param cycle: has pretty print detected a cycle?
        """
        if cycle:
            p.text('Item(...)')
        else:
            p.text('name: {0}'.format(self.name))
            p.breakable()
            p.text('location: {0}'.format(self.location))
            p.breakable()

class WeaponData():
    """
    Class representing weapon data of items
    """
    def __init__(self, damage = None, critical_range = None,
                 critical_damage = None, weapon_type = None,
                 ammunition_type = None):
        """
        Default constructor

        :param damage: array of amount and type of damage
        :type damage: [(integer, string), ...]
        :param critical_range: range where critical might occur
        :type critical_range: integer
        :param critical_damage: multiplier for critical damage
        :type critical_damage: integer
        :param weapon_type: type of weapon
        :type weapon_type: string
        """

        if damage == None:
            self.damage = []
        else:
            self.damage = damage

        self.critical_range = critical_range
        self.critical_damage = critical_damage
        self.weapon_type = weapon_type
        self.ammunition_type = ammunition_type

class ArmourData():
    """
    Represents data of armours

    .. versionadded:: 0.8
    """
    def __init__(self, damage_reduction = None, speed_modifier = None):
        """
        Default constructor

        :param damage_reduction: amount of damage reduced
        :type damage_reduction: int
        :param speed_modifier: modifier for speed of wearer
        :type speed_modifier: double
        """
        super(ArmourData, self).__init__()

        self.damage_reduction = damage_reduction
        self.speed_modifier = speed_modifier

class AmmunitionData():
    """
    Represents data of ammunition

    .. versionadded:: 0.8
    """
    def __init__(self, damage = None, critical_range = None,
                 critical_damage = None, ammunition_type = None, count = 1):
        """
        Default constructor
        """
        super(AmmunitionData, self).__init__()

        if damage == None:
            self.damage = []
        else:
            self.damage = damage

        self.critical_range = critical_range
        self.critical_damage = critical_damage
        self.ammunition_type = ammunition_type
        self.count = count



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

#   Copyright 2010-2013 Tuukka Turto
#
#   This file is part of pyherc.
#
#   pyherc is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#
#   pyherc is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with pyherc.  If not, see <http://www.gnu.org/licenses/>.

"""
module for configuring catacombs
"""
from pyherc.generators.level.room import CatacombsGenerator
from pyherc.generators.level.partitioners import GridPartitioner

from pyherc.generators.level.decorator import ReplacingDecorator
from pyherc.generators.level.decorator import ReplacingDecoratorConfig
from pyherc.generators.level.decorator import WallBuilderDecorator
from pyherc.generators.level.decorator import WallBuilderDecoratorConfig
from pyherc.generators.level.decorator import AggregateDecorator
from pyherc.generators.level.decorator import AggregateDecoratorConfig

from pyherc.generators.level.items import ItemAdderConfiguration, ItemAdder
from pyherc.generators.level.creatures import CreatureAdderConfiguration
from pyherc.generators.level.creatures import CreatureAdder
from pyherc.generators import InventoryConfiguration
from pyherc.generators.level.portals import PortalAdderConfiguration

from pyherc.config.dsl import LevelConfiguration, LevelContext
from pyherc.generators import CreatureConfiguration
from pyherc.ai import FlockingHerbivore, SkeletonWarriorAI
from pyherc.ai.rat import RatAI
from pyherc.ai.firebeetle import FireBeetleAI
from pyherc.data.effects import DamageModifier

def init_level(rng, item_generator, creature_generator, level_size, context):
    """
    Initialise upper catacombs

    :returns: level configuration
    :rtype: LevelConfiguration
    """
    surface_manager = context.surface_manager

    floor_natural = 'natural floor'
    floor_rock = surface_manager.add_icon('floor_rock',
                                          ':rock_floor.png',
                                          '.')

    wall_empty = surface_manager.add_icon('empty wall',
                                          ':transparent.png',
                                          None)
    wall_natural = 'natural wall'
    wall_constructed = 'constructed wall'
    wall_ground = surface_manager.add_icon('wall_ground',
                                           ':ground_wall.png',
                                           ' ')
    wall_rock = surface_manager.add_icon('wall_rock', 
                                         ':rock_wall.png',
                                         '#')
    stairs_down = surface_manager.add_icon('stairs_down',
                                           ':stairs_down.png',
                                           '>')
    stairs_up = surface_manager.add_icon('stairs_up',
                                         ':stairs_up.png',
                                         '<')

    room_generators = [CatacombsGenerator(floor_natural,
                                           wall_empty,
                                           ['upper catacombs',
                                           'lower catacombs'],
                                           rng)]
    level_partitioners = [GridPartitioner(['upper catacombs',
                                           'lower catacombs'],
                                           1,
                                           1,
                                           rng)]

    replacer_config = ReplacingDecoratorConfig(['upper catacombs',
                                                'lower catacombs'],
                                    {floor_natural: floor_rock},
                                    {wall_natural: wall_ground,
                                    wall_constructed: wall_rock})
    replacer = ReplacingDecorator(replacer_config)

    wallbuilder_config = WallBuilderDecoratorConfig(['upper catacombs',
                                                    'lower catacombs'],
                                        {wall_natural: wall_constructed},
                                        wall_empty)
    wallbuilder = WallBuilderDecorator(wallbuilder_config)

    aggregate_decorator_config = AggregateDecoratorConfig(
                                                ['upper catacombs',
                                                'lower catacombs'],
                                                [wallbuilder,
                                                replacer])

    decorators = [AggregateDecorator(aggregate_decorator_config)]

    item_adder_config = ItemAdderConfiguration(['upper catacombs',
                                               'lower catacombs'])
    item_adder_config.add_item(min_amount = 0,
                               max_amount = 4,
                               type = 'weapon',
                               location = 'room')
    item_adder_config.add_item(min_amount = 0,
                               max_amount = 2,
                               type = 'potion',
                               location = 'room')
    item_adder_config.add_item(min_amount = 1,
                               max_amount = 3,
                               type = 'food',
                               location = 'room')
    item_adder_config.add_item(min_amount = 0,
                               max_amount = 2,
                               type = 'armour',
                               location = 'room')
    item_adder_config.add_item(min_amount = 0,
                               max_amount = 1,
                               type = 'tome',
                               location = 'room')
    item_adders = [ItemAdder(item_generator,
                            item_adder_config,
                            rng)]

    creatures_upper = CreatureAdderConfiguration(['upper catacombs'])
    creatures_upper.add_creature(min_amount = 6,
                                 max_amount = 12,
                                 name = 'rat')
    creatures_upper.add_creature(min_amount = 0,
                                 max_amount = 1,
                                 name = 'skeleton warrior')

    creatures_lower = CreatureAdderConfiguration(['lower catacombs'])
    creatures_lower.add_creature(min_amount = 6,
                                 max_amount = 12,
                                 name = 'rat')
    creatures_lower.add_creature(min_amount = 2,
                                 max_amount = 5,
                                 name = 'fire beetle')
    creatures_lower.add_creature(min_amount = 1,
                                 max_amount = 3,
                                 name = 'skeleton warrior')

    creature_adders = [CreatureAdder(creature_generator,
                                    creatures_upper,
                                    rng),
                       CreatureAdder(creature_generator,
                                    creatures_lower,
                                    rng)
                                    ]

    portal_adder_configurations = [PortalAdderConfiguration(
                                        icons = (stairs_down,
                                                 stairs_up),
                                        level_type = 'upper catacombs',
                                        location_type = 'room',
                                        chance = 100,
                                        new_level = 'lower catacombs',
                                        unique = True),
                                    PortalAdderConfiguration(
                                        icons = (stairs_up,
                                                 stairs_down),
                                        level_type = 'upper catacombs',
                                        location_type = 'room',
                                        chance = 100,
                                        new_level = '',
                                        unique = True,
                                        escape_stairs = True)
                                        ]

    level_context = LevelContext(size = level_size,
                                 floor_type = floor_natural,
                                 wall_type = wall_natural,
                                 empty_floor = 0,
                                 empty_wall = wall_empty,
                                 level_types = ['upper catacombs',
                                                'lower catacombs'])

    config = (LevelConfiguration()
                    .with_rooms(room_generators)
                    .with_partitioners(level_partitioners)
                    .with_decorators(decorators)
                    .with_items(item_adders)
                    .with_creatures(creature_adders)
                    .with_portals(portal_adder_configurations)
                    .with_contexts([level_context])
                    .build())

    return config

def init_creatures(context):
    """
    Initialise creatures''

    :returns: list of configuration items
    :rtype: [CreatureConfiguration]
    """
    surface_manager = context.surface_manager
    config = []

    config.append(CreatureConfiguration(name = 'rat',
                                        body = 4,
                                        finesse = 12,
                                        mind = 2,
                                        hp = 2,
                                        speed = 2,
                                        icons = surface_manager.add_icon(
                                                        'rat',
                                                        ':mouse.png', 
                                                        'r', 
                                                        ['yellow', 'dim']),
                                        attack = 1,
                                        ai = RatAI))

    config.append(CreatureConfiguration(name = 'fire beetle',
                                        body = 10,
                                        finesse = 11,
                                        mind = 0,
                                        hp = 4,
                                        speed = 3,
                                        icons = surface_manager.add_icon(
                                                        'fire beetle',
                                                        ':scarab-beetle.png',
                                                        'a',
                                                        ['red']),
                                        attack = 2,
                                        ai = FireBeetleAI))

    skeleton_inventory = [InventoryConfiguration(item_name = 'sword',
                                                min_amount = 0,
                                                max_amount = 1,
                                                probability = 100)]

    skeleton_effects = [DamageModifier(modifier = 2,
                                       damage_type = 'crushing',
                                       duration = None,
                                       frequency = None,
                                       tick = None,
                                       icon = 0,
                                       title = '',
                                       description = ''),
                        DamageModifier(modifier = 2,
                                       damage_type = 'holy',
                                       duration = None,
                                       frequency = None,
                                       tick = None,
                                       icon = 0,
                                       title = '',
                                       description = ''),
                        DamageModifier(modifier = -2,
                                       damage_type = 'piercing',
                                       duration = None,
                                       frequency = None,
                                       tick = None,
                                       icon = 0,
                                       title = '',
                                       description = ''),
                        DamageModifier(modifier = -2,
                                       damage_type = 'poison',
                                       duration = None,
                                       frequency = None,
                                       tick = None,
                                       icon = 0,
                                       title = '',
                                       description = '')]

    config.append(CreatureConfiguration(name = 'skeleton warrior',
                                        body = 8,
                                        finesse = 11,
                                        mind = 0,
                                        hp = 8,
                                        speed = 4,
                                        icons = surface_manager.add_icon(
                                                        'skeleton warrior',
                                                        ':blade-bite.png',
                                                        'Z',
                                                        ['white', 'bold']),
                                        attack = 1,
                                        ai = SkeletonWarriorAI,
                                        inventory = skeleton_inventory,
                                        effects = skeleton_effects))

    return config



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

#   Copyright 2010-2013 Tuukka Turto
#
#   This file is part of pyherc.
#
#   pyherc is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#
#   pyherc is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with pyherc.  If not, see <http://www.gnu.org/licenses/>.

"""
Module for main window related functionality
"""

from herculeum.ui.gui.startgame import StartGameWidget
from herculeum.ui.gui.map import PlayMapWindow
from herculeum.ui.gui.eventdisplay import EventMessageDockWidget
from herculeum.ui.gui.menu import MenuDialog
from herculeum.ui.gui.endscreen import EndScreen
from herculeum.ui.controllers import EndScreenController, StartGameController
from PyQt4.QtGui import QMainWindow, QAction, QIcon
from PyQt4.QtGui import QDialog
from PyQt4.QtGui import QPixmap, QSplashScreen
from PyQt4.QtCore import Qt, QFile
from PyQt4.QtGui import QApplication
import PyQt4.QtGui

class QtUserInterface():
    """
    Class for Qt User Interface

    .. versionadded:: 0.9
    """
    def __init__(self, application):
        """
        Default constructor
        """
        super(QtUserInterface, self).__init__()

        self.application = application
        self.splash_screen = None

        self.qt_app = QApplication([])

    def show_splash_screen(self):
        """
        Show splash screen
        """
        file = QFile(':herculeum.qss')
        file.open(QFile.ReadOnly)
        styleSheet = str(file.readAll().data(), 'ascii')
        self.qt_app.setStyleSheet(styleSheet)

        pixmap = QPixmap(':splash.png')
        self.splash_screen = QSplashScreen(pixmap)
        self.splash_screen.show()

    def show_main_window(self):
        """
        Show main window
        """
        main_window = MainWindow(self.application,
                                 self.application.surface_manager,
                                 self.qt_app,
                                 None,
                                 Qt.FramelessWindowHint,
                                 StartGameController(self.application.level_generator_factory,
                                                     self.application.creature_generator,
                                                     self.application.item_generator))

        self.splash_screen.finish(main_window)
        main_window.show_new_game()

        self.qt_app.exec_()

class MainWindow(QMainWindow):
    """
    Class for displaying main window

    .. versionadded:: 0.5
    """
    def __init__(self, application, surface_manager, qt_app, parent, flags,
                 controller):
        """
        Default constructor
        """
        super(MainWindow, self).__init__(parent, flags)

        self.application = application
        self.surface_manager = surface_manager
        self.qt_app = qt_app
        self.controller = controller

        self.__set_layout()

    def __set_layout(self):

        exit_action = QAction(QIcon(':exit-game.png'),
                             '&Quit',
                             self)

        exit_action.setShortcut('Ctrl+Q')
        exit_action.setStatusTip('Quit game')
        exit_action.triggered.connect(PyQt4.QtGui.qApp.quit)

        inventory_action = QAction(QIcon(':inventory.png'),
                                         'Inventory',
                                         self)
        inventory_action.setShortcut('Ctrl+I')
        inventory_action.setStatusTip('Show inventory')
        inventory_action.triggered.connect(self.__show_menu)

        character_action = QAction(QIcon(':character.png'),
                                         'Character',
                                         self)
        character_action.setShortcut('Ctrl+C')
        character_action.setStatusTip('Show character')

        menubar = self.menuBar()
        file_menu = menubar.addMenu('&File')
        file_menu.addAction(exit_action)

        self.map_window = PlayMapWindow(parent = None,
                                        model = self.application.world,
                                        surface_manager = self.surface_manager,
                                        action_factory = self.application.action_factory,
                                        rng = self.application.rng,
                                        rules_engine = self.application.rules_engine,
                                        configuration = self.application.config)
        self.setCentralWidget(self.map_window)

        self.map_window.MenuRequested.connect(self.__show_menu)
        self.map_window.EndScreenRequested.connect(self.__show_end_screen)

        self.setGeometry(50, 50, 800, 600)
        self.setWindowTitle('Herculeum')
        self.setWindowIcon(QIcon(':rune-stone.png'))
        self.showMaximized()

    def show_new_game(self):
        """
        Show new game dialog
        """
        app = self.application

        start_dialog = StartGameWidget(generator = app.player_generator,
                                       config = self.application.config.controls,
                                       parent = self,
                                       application = self.application,
                                       surface_manager = self.surface_manager,
                                       flags = Qt.Dialog | Qt.CustomizeWindowHint)

        result = start_dialog.exec_()

        if result == QDialog.Accepted:
            player = start_dialog.player_character

            self.controller.setup_world(self.application.world,
                                        player)

            player.register_for_updates(self.map_window.hit_points_widget)
            self.map_window.hit_points_widget.show_hit_points(player)

            self.__show_map_window()

    def __show_map_window(self):
        """
        Show map window
        """
        self.map_window.construct_scene()

    def __show_message_window(self, character):
        """
        Show message display

        :param character: character which events to display
        :type character: Character
        """
        messages_display = EventMessageDockWidget(self, character)

        self.addDockWidget(Qt.BottomDockWidgetArea,
                           messages_display)


    def __show_menu(self):
        """
        Show menu
        """
        menu_dialog = MenuDialog(self.surface_manager,
                                 self.application.world.player,
                                 self.application.action_factory,
                                 self.application.config.controls,
                                 self,
                                 Qt.Dialog | Qt.CustomizeWindowHint)
        menu_dialog.exec_()

    def __show_end_screen(self):
        """
        Show end screen

        .. versionadded:: 0.8
        """
        end_screen = EndScreen(self.application.world,
                               self.application.config.controls,
                               self.application.rules_engine.dying_rules,
                               self,
                               Qt.Dialog | Qt.CustomizeWindowHint,
                               controller = EndScreenController())

        end_screen.exec_()
        self.qt_app.quit()




#!/usr/bin/env python3
# -*- coding: utf-8 -*-

#   Copyright 2010-2013 Tuukka Turto
#
#   This file is part of pyherc.
#
#   pyherc is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#
#   pyherc is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with pyherc.  If not, see <http://www.gnu.org/licenses/>.

"""
Module for inventory actions
"""
class InventoryController():
    """
    Class for inventory actions

    .. versionadded:: 0.9
    """
    def __init__(self, character, action_factory):
        """
        Default constructor
        """
        super(InventoryController, self).__init__()

        self.character = character
        self.action_factory = action_factory

    def use_item(self, item):
        """
        Use item in different ways, depending on the item
        """
        if item.get_main_type() == 'potion':
            self.character.drink(item, self.action_factory)
        elif item.get_main_type() == 'weapon':
            if self.character.inventory.weapon != None:
                self.character.unequip(self.character.inventory.weapon,
                                       self.action_factory)
            self.character.equip(item,
                                 self.action_factory)
        elif item.get_main_type() == 'armour':
            if self.character.inventory.armour != None:
                self.character.unequip(self.character.inventory.armour,
                                       self.action_factory)
            self.character.equip(item,
                                 self.action_factory)
        elif item.get_main_type() == 'ammunition':
            self.character.equip(item,
                                 self.action_factory)

    def unequip_item(self, item):
        """
        Unequip item
        """
        self.character.unequip(item,
                               self.action_factory)

    def pick_up_item(self, item):
        """
        Pick up item
        """
        self.character.pick_up(item, self.action_factory)

    def drop_item(self, item):
        """
        Drop item
        """
        self.character.drop_item(item, self.action_factory)

    def item_description(self, item):
        """
        Get description of an item
        """
        item_description = item.get_name(self.character,
                                         True)
        if item.weapon_data != None:
            data = item.weapon_data
            item_description += '\ndamage: '
            item_description += ' / '.join((str(x[0]) for x in data.damage))
            item_description += ' ('
            item_description += ' / '.join((str(x[1]) for x in data.damage))
            item_description += ')\n'
        elif item.armour_data != None:
            data = item.armour_data
            item_description += '\ndamage reduction: {0}\n'.format(data.damage_reduction)
            item_description += 'speed modifier: {0}\n'.format(data.speed_modifier)

        item_description += '\n' + item.description

        return item_description



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

#   Copyright 2010-2013 Tuukka Turto
#
#   This file is part of pyherc.
#
#   pyherc is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#
#   pyherc is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with pyherc.  If not, see <http://www.gnu.org/licenses/>.

#   explanation written by Björn Bergström of the FOV algorithm used here can be found from:
#   http://roguebasin.roguelikedevelopment.org/index.php/FOV_using_recursive_shadowcasting
#
#   original implementation by Eric D. Burgess is from:
#   http://roguebasin.roguelikedevelopment.org/index.php/Python_shadowcasting_implementation

"""
Line of sight implementation
"""
mult = [
                [1,  0,  0, -1, -1,  0,  0,  1],
                [0,  1, -1,  0,  0, -1,  1,  0],
                [0,  1,  1,  0,  0, -1, -1,  0],
                [1,  0,  0,  1, -1,  0,  0, -1]
            ]

def is_blocked(loc_x, loc_y, level, character = None):
    """
    Checks if given location should be considered blocking for character

    Args:
        loc_x: x-coordinate on the map
        loc_y: y-coordinate on the map
        level: level
        character: character

    Returns:
        False if not blocking, otherwise True
    """
    assert level != None

    return level.blocks_los(loc_x, loc_y)

def cast_light(cx, cy, row, start, end, radius, xx, xy, yx, yy, fov_matrix, level):
    """
    Recursive lightcasting function

    Returns:
        fov_matrix
    """
    if start < end:
        return
    radius_squared = radius*radius
    for j in range(row, radius+1):
        dx, dy = -j-1, -j
        blocked = False
        while dx <= 0:
            dx += 1
            # Translate the dx, dy coordinates into map coordinates:
            X, Y = cx + dx * xx + dy * xy, cy + dx * yx + dy * yy
            # l_slope and r_slope store the slopes of the left and right
            # extremities of the square we're considering:
            l_slope, r_slope = (dx-0.5)/(dy+0.5), (dx+0.5)/(dy-0.5)
            if start < r_slope:
                continue
            elif end > l_slope:
                break
            else:
                # Our light beam is touching this square; light it:
                if dx*dx + dy*dy < radius_squared:
                    #transform from map to light_matrix
                    fov_matrix[X][Y] = True
                if blocked:
                    # we're scanning a row of blocked squares:
                    if is_blocked(X, Y, level):
                        new_start = r_slope
                        continue
                    else:
                        blocked = False
                        start = new_start
                else:
                    if is_blocked(X, Y, level) and j < radius:
                        # This is a blocking square, start a child scan:
                        blocked = True
                        cast_light(cx, cy, j+1, start, l_slope,
                                            radius, xx, xy, yx, yy, fov_matrix, level)
                        new_start = r_slope
        # Row is scanned; do next row unless last square was blocked:
        if blocked:
            break

    return fov_matrix

def do_fov(x, y, radius, fov_matrix, level):
    """
    Calculate lit squares from the given location and radius
    """
    for oct in range(8):
        cast_light(x, y, 1, 1.0, 0.0, radius,
                            mult[0][oct], mult[1][oct],
                            mult[2][oct], mult[3][oct], fov_matrix, level)

    return fov_matrix

def get_fov_matrix(model, character, distance = 7):
    """
    Calculate field of vision matrix

    Args:
        model: Model to use
        character: Character to calculate matrix for
        distance: maximum view distance

    Returns:
        Matrix defining which parts are visible and which not
    """
    fov_matrix = []

    width = len(character.level.walls[0])
    height = len(character.level.walls)

    for i in range(height):
        fov_matrix.append([False] * width)

    fov_matrix[character.location[0]][character.location[1]] = True

    return do_fov(character.location[0],
                  character.location[1], distance,
                  fov_matrix, character.level)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

#   Copyright 2010-2013 Tuukka Turto
#
#   This file is part of pyherc.
#
#   pyherc is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#
#   pyherc is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with pyherc.  If not, see <http://www.gnu.org/licenses/>.

"""
Module defining classes related to inventory actions
"""
from pyherc.aspects import logged
from pyherc.events import PickUpEvent, DropEvent

class PickUpAction():
    """
    Action for picking up item

    .. versionadded:: 0.4
    """
    @logged
    def __init__(self, character, item):
        """
        Default constructor

        :param character: character moving
        :type character: Character
        :param item: item to pick up
        :type item: Item
        """
        super(PickUpAction, self).__init__()
        self.character = character
        self.item = item

    @logged
    def execute(self):
        """
        Executes this action
        """
        if self.is_legal():
            self.character.level.items.remove(self.item)

            if not self._merge_similar_items():
                self.character.inventory.append(self.item)
                self.item.location = ()

            self.character.raise_event(PickUpEvent(self.character,
                                                   self.item))

        self.character.add_to_tick(2)

    @logged
    def _merge_similar_items(self):
        """
        Merge similar items in character inventory

        :returns: True if item was merged, otherwise False
        :rtype: boolean
        """
        if self.item.ammunition_data == None:
            return False

        items = [x for x in self.character.inventory
                 if x.name == self.item.name]

        if len(items) == 0:
            return False

        merged_data = items[0].ammunition_data
        merged_data.count = merged_data.count + self.item.ammunition_data.count

        return True

    @logged
    def is_legal(self):
        """
        Check if the action is possible to perform

        :returns: True if move is possible, false otherwise
        :rtype: Boolean
        """
        character = self.character
        item = self.item

        if character.location != item.location:
            return False

        return True

class DropAction():
    """
    Action for dropping item

    .. versionadded:: 0.5
    """
    @logged
    def __init__(self, character, item):
        """
        Default constructor

        :param character: character dropping item
        :type character: Character
        :param item: item to drop
        :type item: Item
        """
        super(DropAction, self).__init__()

        self.character = character
        self.item = item

    @logged
    def execute(self):
        """
        Executes this action
        """
        self.character.inventory.remove(self.item)
        self.character.level.add_item(item = self.item,
                                      location = (self.character.location[0],
                                                  self.character.location[1]))
        self.character.add_to_tick(2)
        self.character.raise_event(DropEvent(self.character,
                                             self.item))

    @logged
    def is_legal(self):
        """
        Check if the action is possible to perform

        :returns: True if move is possible, false otherwise
        :rtype: Boolean
        """
        return True



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

#   Copyright 2010-2013 Tuukka Turto
#
#   This file is part of pyherc.
#
#   pyherc is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#
#   pyherc is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with pyherc.  If not, see <http://www.gnu.org/licenses/>.

"""
Module defining classes related to AttackAction
"""
import random
from pyherc.aspects import logged
from pyherc.events import AttackHitEvent, AttackNothingEvent, AttackMissEvent

class AttackAction():
    """
    Action for attacking
    """
    @logged
    def __init__(self, attack_type, to_hit, damage,
                 attacker, target, effect_factory, dying_rules,
                 additional_rules):
        """
        Default constructor

        :param attack_type: type of the attack
        :type attack_type: string
        :param to_hit: ToHit object for calculating if attack hits
        :type to_hit: ToHit
        :param damage: Damage object for calculating done damage
        :type damage: Damage
        :param attacker: Character doing attack
        :type attacker: Character
        :param target: Character being attacked
        :type target: Character
        :param effect_factory: Factory used for creating magic effects
        :type effect_factory: EffectFactory
        :param dying_rules: rules for dying
        :param additional_rules: additional rules
        :type additional_rules: AdditionalRules
        """
        self.action_type = 'attack'
        self.attack_type = attack_type
        self.to_hit = to_hit
        self.damage = damage
        self.attacker = attacker
        self.target = target
        self.effect_factory = effect_factory
        self.dying_rules = dying_rules
        self.additional_rules = additional_rules

    @logged
    def is_legal(self):
        """
        Check if the attack is possible to perform

        :returns: true if attackis possible, false otherwise
        :rtype: Boolean
        """
        if self.attacker == None:
            return False

        return True

    @logged
    def execute(self):
        """
        Executes this Attack
        """
        if self.target == None:
            self.attacker.raise_event(AttackNothingEvent(
                                            attacker = self.attacker,
                                            affected_tiles = []))
        else:

            was_hit = self.to_hit.is_hit()

            if was_hit:
                self.damage.apply_damage(self.target)

                self.attacker.raise_event(AttackHitEvent(
                                          type = self.attack_type,
                                          attacker = self.attacker,
                                          target = self.target,
                                          damage = self.damage,
                                          affected_tiles = [self.target.location]))

                self.__trigger_attack_effects()
            else:
                self.attacker.raise_event(AttackMissEvent(
                                          type = self.attack_type,
                                          attacker = self.attacker,
                                          target = self.target,
                                          affected_tiles = [self.target.location]))

            self.dying_rules.check_dying(self.target)

        self.additional_rules.after_attack()
        self.attacker.add_to_tick(3)

    @logged
    def __trigger_attack_effects(self):
        """
        Trigger effects

        .. versionadded:: 0.4
        """
        weapon = self.attacker.inventory.weapon
        effects = self.attacker.get_effect_handles('on attack hit')

        if weapon != None:
            effects.extend(weapon.get_effect_handles('on attack hit'))
        for effect_spec in effects:
            effect = self.effect_factory.create_effect(
                                                    effect_spec.effect,
                                                    target = self.target)

            if not effect.duration or effect.duration <= 0:
                effect.trigger(self.dying_rules)
            else:
                self.target.add_effect(effect)

class ToHit():
    """
    Checks done for hitting
    """
    @logged
    def __init__(self, attacker,  target,
                        random_number_generator = random.Random()):
        """
        Default constructor
        """
        self.attacker = attacker
        self.target = target
        self.rng = random_number_generator

    @logged
    def is_hit(self):
        """
        Checks if the hit lands
        @returns: True if hit is successful, False otherwise
        """
        return True

class Damage():
    """
    Damage done in attack
    """
    @logged
    def __init__(self, damage):
        """
        Default constructor
        """
        self.__damage = damage
        self.damage_inflicted = 0

    @logged
    def apply_damage(self, target):
        """
        Applies damage to target
        :param target: target to damage
        :type target: Character
        """
        for damage in self.__damage:
            damage_type = damage[1]

            matching_modifiers = [x for x in target.get_effects()
                                  if x.effect_name == 'damage modifier'
                                  and x.damage_type == damage_type]

            self.damage_inflicted = (self.damage_inflicted +
                                     damage[0] +
                                     sum(x.modifier for x in matching_modifiers))

            if self.damage_inflicted < 1:
                self.damage_inflicted = 0

        if target.inventory.armour != None:
            armour = target.inventory.armour.armour_data.damage_reduction
        else:
            armour = 0

        if armour < self.damage_inflicted:
            self.damage_inflicted = self.damage_inflicted - armour
        else:
            if armour < self.damage_inflicted * 2:
                self.damage_inflicted = 1
            else:
                self.damage_inflicted = 0

        target.hit_points = target.hit_points - self.damage_inflicted

    def __get_damage(self):
        """
        Total damage caused

        :returns: total damage caused
        :rtype: int
        """
        return sum(x[0] for x in self.__damage)

    def __get_damage_types(self):
        """
        Types of damage caused

        :return: types of damage caused
        :rtype: [string]
        """
        return [x[1] for x
                in self.__damage]

    damage = property(__get_damage)
    damage_types = property(__get_damage_types)

class AdditionalRules():
    """
    Additional rules for attacks

    .. versionadded: 0.8
    """
    def __init__(self, attacker):
        """
        Default constructor
        """
        super(AdditionalRules, self).__init__()

    def after_attack(self):
        """
        Processing happening after an attack
        """
        pass



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

#   Copyright 2010-2013 Tuukka Turto
#
#   This file is part of pyherc.
#
#   pyherc is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#
#   pyherc is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with pyherc.  If not, see <http://www.gnu.org/licenses/>.

"""
Classes for move events
"""
from pyherc.events.event import Event

class AttackHitEvent(Event):
    """
    Event that can be used to indicate attacking hitting

    .. versionadded:: 0.4
    """
    def __init__(self, type, attacker, target, damage, affected_tiles):
        """
        Default constructor
        """
        super(AttackHitEvent, self).__init__(event_type = 'attack hit',
                                             level = attacker.level,
                                             location = attacker.location,
                                             affected_tiles = affected_tiles)

        self.type = type
        self.attacker = attacker
        self.target = target
        self.damage = damage

    def get_description(self, point_of_view):
        """
        Description of the event

        :param point_of_view: point of view for description
        :type point_of_view: Character
        :returns: description of the event
        :rtype: string
        """
        if point_of_view == self.attacker:
            description = 'You hit {0} ({1} points of damage)'.format(self.target.name,
                                                     self.damage.damage_inflicted)
        elif point_of_view == self.target:
            description = '{0} hits you ({1} points of damage)'.format(self.attacker.name,
                                                      self.damage.damage_inflicted)
        else:
            description = '{0} hits {1} ({2} points of damage)'.format(
                            self.attacker.name,
                            self.target.name,
                            self.damage.damage_inflicted)

        return description

class AttackMissEvent(Event):
    """
    Event that can be used to indicate attacking missing

    .. versionadded:: 0.4
    """
    def __init__(self, type, attacker, target, affected_tiles):
        """
        Default constructor
        """
        super(AttackMissEvent, self).__init__(event_type = 'attack miss',
                                              level = attacker.level,
                                              location = attacker.location,
                                              affected_tiles = affected_tiles)

        self.type = type
        self.attacker = attacker
        self.target = target

    def get_description(self, point_of_view):
        """
        Description of the event

        :param point_of_view: point of view for description
        :type point_of_view: Character
        :returns: description of the event
        :rtype: string
        """
        if point_of_view == self.attacker:
            description = 'You miss {0}'.format(self.target.name)
        elif point_of_view == self.target:
            description = '{0} misses you'.format(self.attacker.name)
        else:
            description = '{0} misses {1}'.format(self.attacker.name,
                                                  self.target.name)

        return description

class AttackNothingEvent(Event):
    """
    Event describing attack that targets nothing at all

    .. versionadded:: 0.5
    """
    def __init__(self, attacker, affected_tiles):
        """
        Default constructor
        """
        super(AttackNothingEvent, self).__init__(
                                        event_type = 'attack nothing',
                                        level = attacker.level,
                                        location = attacker.location,
                                        affected_tiles = affected_tiles)
        self.attacker = attacker

    def get_description(self, point_of_view):
        """
        Description of the event

        :param point_of_view: point of view for description
        :type point_of_view: Character
        :returns: description of the event
        :rtype: string
        """
        if point_of_view == self.attacker:
            description = 'You flail around, hitting nothing'
        else:
            description = '{0} flails around, hitting nothing'.format(
                                                            self.attacker.name)

        return description



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

#   Copyright 2010-2013 Tuukka Turto
#
#   This file is part of pyherc.
#
#   pyherc is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#
#   pyherc is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with pyherc.  If not, see <http://www.gnu.org/licenses/>.

"""
Module for partitioning level to equal grid
"""

import logging
from pyherc.generators.level.partitioners.section import Section
from pyherc.aspects import logged

class RandomConnector():
    """
    Class for building random connection network from sections
    """
    @logged
    def __init__(self, random_generator):
        """
        Default constructor

        :param random_generator: random number generator
        :type random_generator: Random
        """
        self.random_generator = random_generator
        self.logger = logging.getLogger('pyherc.generators.level.partitioners.grid.RandomConnector') #pylint: disable=C0301

    @logged
    def connect_sections(self, sections, start_section = None):
        """
        Connects sections together

        :param sections: sections to connect
        :type sections: [Section]
        :param start_section: optional parameter specifying starting section
        :type start_section: Section
        """
        if start_section == None:
            start_location = self.random_generator.choice(sections)
        else:
            start_location = start_section

        self.form_path_from_sections(start_location, sections)

        if len(sections) > 1:
            unconnected_sections = [x for x in sections
                                if x.connected == False]

            while len(unconnected_sections) > 0:
                edge_sections = [x for x in sections
                                if x.connected == True
                                and x.has_unconnected_neighbours()]

                start_location = self.random_generator.choice(edge_sections)

                self.form_path_from_sections(start_location, sections)

                unconnected_sections = [x for x in sections
                                        if x.connected == False]

        return sections

    @logged
    def form_path_from_sections(self, start_section, sections):
        """
        Builds path of connected sections

        :param start_section: section to start connecting from
        :type start_section: Section
        :param sections: sections to connect
        :type sections: [Section]
        """
        current_section = start_section
        unconnected_neighbours = [x for x in current_section.neighbours
                                                if x.connected == False]

        while len(unconnected_neighbours) > 0:
            next_section = self.random_generator.choice(
                                                unconnected_neighbours)

            current_section.connect_to(next_section)

            current_section = next_section
            unconnected_neighbours = [x for x in current_section.neighbours
                                                if x.connected == False]


class GridPartitioner():
    """
    Class for partitioning level to equal grid
    """
    @logged
    def __init__(self, level_types, x_sections,  y_sections, random_generator):
        """
        Default constructor

        :param level_types: types of level partitioner can be used for
        :type level_types: [string]
        :param x_sections: amount of sections to split horizontally
        :type x_sections: integer
        :param y_sections: amount of sections to split vertically
        :type y_sections: integer
        :param random_generator: random number generator
        :type random_generator: Random
        """
        self.connectors = [RandomConnector(random_generator)]
        self.level_types = level_types
        self.x_sections = x_sections
        self.y_sections = y_sections
        self.random_generator = random_generator

    @logged
    def partition_level(self, level):
        """
        Creates partitioning for a given level with connection points

        :param level: level to partition
        :type level: Level
        :returns: connected sections
        :rtype: [Section]
        """
        sections = []
        section_matrix = [[None for i in range(self.y_sections)]
                                               for j in range(self.x_sections)]
        size_of_level = level.get_size()

        x_sections = self.split_range_to_equals(size_of_level[0],
                                                self.x_sections)
        y_sections = self.split_range_to_equals(size_of_level[1],
                                                self.y_sections)

        for y_block in range(len(y_sections)):
            for x_block in range(len(x_sections)):
                temp_section = Section(
                                       (x_sections[x_block][0],
                                        y_sections[y_block][0]),
                                        (x_sections[x_block][1],
                                        y_sections[y_block][1]),
                                        level,
                                        self.random_generator)

                self.connect_new_section(temp_section,
                                         (x_block, y_block),
                                         section_matrix)
                section_matrix[x_block][y_block] = temp_section
                sections.append(temp_section)

        connector = self.random_generator.choice(self.connectors)
        connected_sections = connector.connect_sections(sections)

        return connected_sections

    @logged
    def connect_new_section(self, section, location, sections):
        """
        Connects section in given location to its neighbours

        :param section: section to connect
        :type section: Section
        :param location: location of the section
        :param sections: sections
        :type sections: [Section]
        """
        if location[0] > 0:
            left_section = sections[location[0]-1][location[1]]
            left_section.neighbours.append(section)
            section.neighbours.append(left_section)

        if location[1] > 0:
            up_section = sections[location[0]][location[1]-1]
            up_section.neighbours.append(section)
            section.neighbours.append(up_section)

    @logged
    def split_range_to_equals(self, length, sections):
        """
        Split range into equal sized chunks

        :param length: range to split
        :type length: integer
        :param sections: amount of sections to split
        :type sections: integer
        :returns: list containing end points of chunks
        :rtype: [integer]
        """
        section_length = length // sections
        ranges = []

        for i in range(sections):
            ranges.append((i * section_length, (i + 1) * section_length - 1))

        return ranges



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

#   Copyright 2010-2013 Tuukka Turto
#
#   This file is part of pyherc.
#
#   pyherc is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#
#   pyherc is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with pyherc.  If not, see <http://www.gnu.org/licenses/>.

"""
Module displaying events on screen
"""
from PyQt4.QtGui import QTextEdit, QWidget, QHBoxLayout, QDockWidget

class EventMessageDockWidget(QDockWidget):
    """
    Dock widget to display event messages

    .. versionadded:: 0.5
    """
    def __init__(self, parent, character):
        """
        Default constructor
        """
        super(EventMessageDockWidget, self).__init__(parent)

        self.character = character

        self.__set_layout(character)

    def __set_layout(self, character):
        """
        Set layout of this widget
        """
        self.message_display = EventMessageWidget(self)
        self.setWidget(self.message_display)
        self.setWindowTitle('Messages')

        character.register_event_listener(self.message_display)
        self.message_display.set_point_of_view(character)


class EventMessageWidget(QWidget):
    """
    Widget to display event messages

    .. versionadded:: 0.5
    """
    def __init__(self, parent):
        """
        Default constructor
        """
        super(EventMessageWidget, self).__init__(parent)

        self.__set_layout()
        self.__set_events_to_display()

    def __set_layout(self):
        """
        Set layout of this component
        """
        self.horizontal_layout = QHBoxLayout()

        self.text_edit = QTextEdit()
        self.text_edit.setReadOnly(True)
        self.text_edit.setText('You set out to adventure')

        self.horizontal_layout.addWidget(self.text_edit)

        self.setLayout(self.horizontal_layout)

    def __set_events_to_display(self):
        """
        Configure events to display
        """
        self.event_types_to_show = ['attack hit',
                                    'attack miss',
                                    'attack nothing',
                                    'poison triggered',
                                    'poison ended',
                                    'poisoned',
                                    'heal started',
                                    'heal ended',
                                    'heal triggered',
                                    'death',
                                    'pick up',
                                    'drop',
                                    'damage triggered',
                                    'equip',
                                    'unequip', 
                                    'error']

    def set_point_of_view(self, character):
        """
        Set point of view for events to display
        """
        self.character = character

    def receive_event(self, event):
        """
        Receive event to display
        """
        if event.event_type in self.event_types_to_show:
            self.text_edit.append(event.get_description(self.character))



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

#   Copyright 2010-2013 Tuukka Turto
#
#   This file is part of pyherc.
#
#   pyherc is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#
#   pyherc is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with pyherc.  If not, see <http://www.gnu.org/licenses/>.

"""
Module for moving related actions
"""

import logging

__logger = logging.getLogger('pyherc.rules.moving')

def deactivate(model, character):
    """
    Deactivates a character, causing it to hide
    @param model: model to use
    @param character: character to hide
    """
    item = character.get_mimic_item()
    character.set_mimic_item(None)
    level = character.level
    location = character.get_location()

    level.add_item(item, location)
    level.remove_creature(character)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

#   Copyright 2010-2013 Tuukka Turto
#
#   This file is part of pyherc.
#
#   pyherc is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#
#   pyherc is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with pyherc.  If not, see <http://www.gnu.org/licenses/>.

"""
module for configuring upper crypt
"""
from pyherc.generators.level.partitioners import GridPartitioner
from pyherc.generators.level.room import SquareRoomGenerator, PillarRoomGenerator

from pyherc.generators.level.decorator import ReplacingDecorator
from pyherc.generators.level.decorator import ReplacingDecoratorConfig
from pyherc.generators.level.decorator import WallBuilderDecorator
from pyherc.generators.level.decorator import WallBuilderDecoratorConfig
from pyherc.generators.level.decorator import AggregateDecorator
from pyherc.generators.level.decorator import AggregateDecoratorConfig

from pyherc.generators.level.items import ItemAdderConfiguration, ItemAdder
from pyherc.generators.level.creatures import CreatureAdderConfiguration
from pyherc.generators.level.creatures import CreatureAdder

from pyherc.generators.level.portals import PortalAdderConfiguration

from pyherc.ai import FlockingHerbivore
from pyherc.generators import CreatureConfiguration
from pyherc.data.effects import EffectHandle

from pyherc.config.dsl import LevelConfiguration, LevelContext

def init_level(rng, item_generator, creature_generator, level_size, context):
    """
    Initialise upper crypt levels

    :returns: level configuration
    :rtype: LevelConfiguration
    """
    surface_manager = context.surface_manager

    floor_natural = 'natural floor'
    floor_rock = surface_manager.add_icon('floor_rock', ':rock_floor.png', '.')
    floor_constructed = 'constructed floor'
    floor_tiled = surface_manager.add_icon('floor_tiled', ':tiled_floor.png', '.')

    wall_empty = surface_manager.add_icon('empty wall', ':transparent.png', None)
    wall_natural = 'natural wall'
    wall_constructed = 'constructed wall'
    wall_ground = surface_manager.add_icon('wall_ground', ':ground_wall.png', ' ')
    wall_rock = surface_manager.add_icon('wall_rock', ':rock_wall.png', '#')
    pillar = surface_manager.add_icon('pillar', ':pillar.png', '#')

    stairs_down = surface_manager.add_icon('stairs_down', ':stairs_down.png', '>')
    stairs_up = surface_manager.add_icon('stairs_up', ':stairs_up.png', '<')

    room_generators = [SquareRoomGenerator(floor_natural,
                                           wall_empty,
                                           ['upper crypt']),
                       SquareRoomGenerator(floor_constructed,
                                           wall_empty,
                                           ['upper crypt']),
                       PillarRoomGenerator(floor_tile = floor_constructed,
                                           empty_tile = wall_empty,
                                           pillar_tile = pillar,
                                           level_types = ['upper crypt'])
                                           ]
    level_partitioners = [GridPartitioner(['upper crypt'],
                                          4,
                                          2,
                                          rng)]

    replacer_config = ReplacingDecoratorConfig(['upper crypt'],
                                    {floor_natural: floor_rock,
                                    floor_constructed: floor_tiled},
                                    {wall_natural: wall_ground,
                                    wall_constructed: wall_rock})
    replacer = ReplacingDecorator(replacer_config)

    wallbuilder_config = WallBuilderDecoratorConfig(['upper crypt'],
                                        {wall_natural: wall_constructed},
                                        wall_empty)
    wallbuilder = WallBuilderDecorator(wallbuilder_config)

    aggregate_decorator_config = AggregateDecoratorConfig(['upper crypt'],
                                                          [wallbuilder,
                                                          replacer])

    decorators = [AggregateDecorator(aggregate_decorator_config)]

    item_adder_config = ItemAdderConfiguration(['upper crypt'])
    item_adder_config.add_item(min_amount = 2,
                               max_amount = 4,
                               type = 'weapon',
                               location = 'room')
    item_adder_config.add_item(min_amount = 2,
                               max_amount = 4,
                               type = 'potion',
                               location = 'room')
    item_adder_config.add_item(min_amount = 0,
                               max_amount = 5,
                               type = 'food',
                               location = 'room')
    item_adder_config.add_item(min_amount = 0,
                               max_amount = 1,
                               type = 'tome',
                               location = 'room')
    item_adders = [ItemAdder(item_generator,
                            item_adder_config,
                            rng)]

    creature_adder_config = CreatureAdderConfiguration(['upper crypt'])

    creature_adder_config.add_creature(min_amount = 4,
                                       max_amount = 8,
                                       name = 'spider')

    creature_adders = [CreatureAdder(creature_generator,
                                    creature_adder_config,
                                    rng)]

    portal_adder_configurations = [PortalAdderConfiguration(
                                        icons = (stairs_down,
                                                 stairs_up),
                                        level_type = 'upper catacombs',
                                        location_type = 'room',
                                        chance = 10,
                                        new_level = 'upper crypt',
                                        unique = True)]

    level_context = LevelContext(size = level_size,
                                 floor_type = floor_natural,
                                 wall_type = wall_natural,
                                 empty_floor = 0,
                                 empty_wall = wall_empty,
                                 level_types = ['upper crypt',
                                                'lower crypt'])

    config = (LevelConfiguration()
                    .with_rooms(room_generators)
                    .with_partitioners(level_partitioners)
                    .with_decorators(decorators)
                    .with_items(item_adders)
                    .with_creatures(creature_adders)
                    .with_portals(portal_adder_configurations)
                    .with_contexts([level_context])
                    .build())
    return config

def init_creatures(context):
    """
    Initialise creatures

    :returns: list of creature configurations
    :rtype: [CreatureConfiguration]
    """
    config = []
    surface_manager = context.surface_manager

    config.append(CreatureConfiguration(name = 'spider',
                                        body = 6,
                                        finesse = 12,
                                        mind = 8,
                                        hp = 6,
                                        speed = 1,
                                        icons = surface_manager.add_icon('spider', ':masked-spider.png', 's', ['white', 'dim']),
                                        attack = 4,
                                        ai = FlockingHerbivore,
                                        effect_handles = [EffectHandle(
                                                    trigger = 'on attack hit',
                                                    effect = 'minor poison',
                                                    parameters = None,
                                                    charges = 100)]))

    return config



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

#   Copyright 2010-2013 Tuukka Turto
#
#   This file is part of pyherc.
#
#   pyherc is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#
#   pyherc is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with pyherc.  If not, see <http://www.gnu.org/licenses/>.

"""
Module for displaying character
"""
from PyQt4.QtGui import QWidget, QLabel, QHBoxLayout, QVBoxLayout
from PyQt4.QtGui import QGridLayout
from PyQt4.QtSvg import QSvgWidget

from herculeum.ui.gui.widgets import ListView

class CharacterWidget(QWidget):
    """
    Widget to show character

    .. versionadded:: 0.7
    """
    def __init__(self, surface_manager, character, parent):
        """
        Default constructor
        """
        super(CharacterWidget, self).__init__(parent)

        self.body = None
        self.finesse = None
        self.character_icon = None
        self.hp = None
        self.mind = None
        self.mana = None

        self.__set_layout(surface_manager,
                          character)

    def __set_layout(self, surface_manager, character):
        """
        Set layout of this widget
        """
        main_layout = QHBoxLayout()
        left_layout = QVBoxLayout()
        right_layout = QVBoxLayout()

        left_top_layout = QHBoxLayout()
        left_bottom_layout = QVBoxLayout()

        icon_layout = QVBoxLayout()
        stat_layout = QGridLayout()

        self.body = QLabel()
        self.body.setText(str(character.body))
        self.body.setObjectName('no_border')
        self.mind = QLabel()
        self.mind.setText(str(character.mind))
        self.mind.setObjectName('no_border')
        self.finesse = QLabel()
        self.finesse.setText(str(character.finesse))
        self.finesse.setObjectName('no_border')
        self.hp = QLabel()
        self.hp.setText('{0}/{1}'.format(character.hit_points,
                                         character.max_hp))
        self.hp.setObjectName('no_border')
        self.mana = QLabel()
        self.mana.setText('0/0')
        self.mana.setObjectName('no_border')

        body_label = QLabel('Body:')
        body_label.setObjectName('no_border')
        mind_label = QLabel('Mind:')
        mind_label.setObjectName('no_border')
        finesse_label = QLabel('Finesse:')
        finesse_label.setObjectName('no_border')
        hp_label = QLabel('HP:')
        hp_label.setObjectName('no_border')
        mana_label = QLabel('Mana:')
        mana_label.setObjectName('no_border')

        stat_layout.addWidget(self.body, 0, 1)
        stat_layout.addWidget(body_label, 0, 0)
        stat_layout.addWidget(self.mind, 1, 1)
        stat_layout.addWidget(mind_label, 1, 0)
        stat_layout.addWidget(self.finesse, 2, 1)
        stat_layout.addWidget(finesse_label, 2, 0)
        stat_layout.addWidget(self.hp, 3, 1)
        stat_layout.addWidget(hp_label, 3, 0)
        stat_layout.addWidget(self.mana, 4, 1)
        stat_layout.addWidget(mana_label, 4, 0)


        self.character_icon = QSvgWidget(':strong.svg')
        self.character_icon.setMaximumSize(150, 150)
        self.character_icon.setMinimumSize(150, 150)
        icon_layout.addWidget(self.character_icon)

        skills_label = QLabel('Skills')
        skills_label.setObjectName('section_title')
        skills = ListView()

        right_layout.addWidget(skills_label)
        right_layout.addWidget(skills)

        effects_label = QLabel('Effects')
        effects_label.setObjectName('section_title')
        effects = ListView()

        for effect in character.get_effects():
            effects.add_item(title = effect.title,
                             description = effect.description,
                             icon = surface_manager.get_icon(effect.icon))

        left_bottom_layout.addWidget(effects_label)
        left_bottom_layout.addWidget(effects)

        left_top_layout.addLayout(icon_layout)
        left_top_layout.addLayout(stat_layout)
        left_layout.addLayout(left_top_layout)
        left_layout.addLayout(left_bottom_layout)
        main_layout.addLayout(left_layout)
        main_layout.addLayout(right_layout)

        self.setLayout(main_layout)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

#   Copyright 2010-2013 Tuukka Turto
#
#   This file is part of pyherc.
#
#   pyherc is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#
#   pyherc is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with pyherc.  If not, see <http://www.gnu.org/licenses/>.

"""
Module for herculeum main configuration
"""
import pyherc.config

class Configuration(pyherc.config.Configuration):
    """
    Configuration for herculeum
    """

    def __init__(self, model, config_package, controls, surface_manager):
        """
        Default constructor

        :param model: model to use in configuration
        :param type: Model
        """
        super(Configuration, self).__init__(model)

        self.surface_manager = surface_manager
        self.surface_manager.load_resources()

        self.context = ConfigurationContext(config_package,
                                            self.surface_manager)

        self.controls = controls

    def initialise(self):
        """
        Initialise configuration
        """
        super(Configuration, self).initialise(self.context)

class ConfigurationContext():
    """
    Class to relay information between different configuration phases

    .. versionadded:: 0.6
    """
    def __init__(self, config_package, surface_manager):
        """
        Default constructor

        :param config_package: package containing configurations
        :type config_package: Package
        :param surface_manager: manager to handle graphics
        :type surface_manager: SurfaceManager
        """
        super(ConfigurationContext, self).__init__()
        self.config_package = config_package
        self.surface_manager = surface_manager



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

#   Copyright 2010-2013 Tuukka Turto
#
#   This file is part of pyherc.
#
#   pyherc is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#
#   pyherc is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with pyherc.  If not, see <http://www.gnu.org/licenses/>.

"""
Classes for heal events
"""
from pyherc.events.event import Event
from pyherc.aspects import logged

class HealTriggeredEvent(Event):
    """
    Event that can be used to relay information about heal being triggered

    .. versionadded:: 0.6
    """

    @logged
    def __init__(self, target, healing):
        """
        Default constructor

        :param target: character who is being healed
        :type target: Character
        :param healing: amount of healing
        :type healing: int
        """
        super(HealTriggeredEvent, self).__init__(event_type = 'heal triggered',
                                                 level = target.level,
                                                 location = target.location,
                                                 affected_tiles = [])
        self.target = target
        self.healing = healing

    def get_description(self, point_of_view):
        """
        Description of the event

        :param point_of_view: point of view for description
        :type point_of_view: Character
        :returns: description of the event
        :rtype: string
        """
        if point_of_view == self.target:
            description = 'You feel better ({0} points healed)'.format(self.healing)
        else:
            description = '{0} feels better ({1} points healed)'.format(self.target.name,
                                                                        self.healing)

        return description

class HealAddedEvent(Event):
    """
    Event raised when character has been cast healing

    .. versionadded:: 0.6
    """

    @logged
    def __init__(self, target, effect):
        """
        Default constructor

        :param target: target of the event
        :type target: Character
        :param effect: healing effect added
        :type effect: Heal
        """
        super(HealAddedEvent, self).__init__(event_type = 'heal started',
                                             level = target.level,
                                             location = target.location,
                                             affected_tiles = [])
        self.target = target
        self.effect = effect

    def get_description(self, point_of_view):
        """
        Description of the event

        :param point_of_view: point of view for description
        :type point_of_view: Character
        :returns: description of the event
        :rtype: string
        """
        if point_of_view == self.target:
            description = 'You are being healed'
        else:
            description = '{0} is being healed'.format(self.target.name)

        return description

class HealEndedEvent(Event):
    """
    Event to signal that healing is over
    """

    @logged
    def __init__(self, target, effect):
        """
        Default constructor

        :param actor: character not being healed anymore
        :type actor: Character
        :param effect: heal effect being removed
        :type effect: Heal
        """
        super(HealEndedEvent, self).__init__(event_type = 'heal ended',
                                             level = target.level,
                                             location = target.location,
                                             affected_tiles = [])
        self.target = target
        self.effect = effect

    def get_description(self, point_of_view):
        """
        Description of the event

        :param point_of_view: point of view for description
        :type point_of_view: Character
        :returns: description of the event
        :rtype: string
        """
        if point_of_view == self.target:
            description = 'You are no longer being healed'
        else:
            description = '{0} is no longer being healed'.format(self.target.name)

        return description



#!/usr/bin/env python3
#!/usr/bin/env python3
# -*- coding: utf-8 -*-

#   Copyright 2010-2013 Tuukka Turto
#
#   This file is part of pyherc.
#
#   pyherc is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#
#   pyherc is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with pyherc.  If not, see <http://www.gnu.org/licenses/>.

"""
module for configuring catacombs
"""
from pyherc.generators import ItemConfiguration, WeaponConfiguration
from pyherc.generators import AmmunitionConfiguration
from pyherc.generators import ArmourConfiguration
from pyherc.data.effects import EffectHandle

def init_items(context):
    """
    Initialise common items

    :returns: item configurations
    :rtype: [ItemConfiguration]
    """
    surface_manager = context.surface_manager
    config = []

    config.append(
                  ItemConfiguration(name = 'apple',
                                    cost = 1,
                                    weight = 1,
                                    icons = [surface_manager.add_icon(
                                                            'apple',
                                                            ':apple.png',
                                                            '%', 
                                                            ['red'])],
                                    types = ['food'],
                                    rarity = 'common'))

    config.append(
                  ItemConfiguration(name = 'dagger',
                                    description = 'Light and simple weapon that does not do much damage. Small size allows skilled wielder to do critical damage easily though.',
                                    cost = 10,
                                    weight = 1,
                                    icons = [surface_manager.add_icon(
                                                            'dagger',
                                                            ':dagger.png',
                                                            ')',
                                                            ['dim', 'cyan'])],
                                    types = ['weapon',
                                               'light weapon',
                                               'melee',
                                               'simple weapon'],
                                    rarity = 'common',
                                    weapon_configration = WeaponConfiguration(
                                            damage = [(2, 'piercing')],
                                            critical_range = 10,
                                            critical_damage = 2,
                                            weapon_class = 'simple')))

    config.append(
                  ItemConfiguration(name = 'sword',
                                    description = 'Staple tool for adventurers and soldiers alike. Swords do more damage than daggers, but are harder to use effectively.',
                                    cost = 20,
                                    weight = 4,
                                    icons = [surface_manager.add_icon(
                                                            'sword',
                                                            ':stiletto.png',
                                                            ')',
                                                            ['dim', 'cyan'])],
                                    types = ['weapon',
                                             'one-handed',
                                             'melee',
                                             'martial weapon'],
                                    rarity = 'uncommon',
                                    weapon_configration = WeaponConfiguration(
                                            damage = [(2, 'piercing'),
                                                      (2, 'slashing')],
                                            critical_range = 12,
                                            critical_damage = 2,
                                            weapon_class = 'martial')))

    config.append(
                  ItemConfiguration(name = 'axe',
                                    description = 'Heavy battle axes can cut through flesh and crush through bones. Not very common weapon, because effective use requires years of training.',
                                    cost = 20,
                                    weight = 4,
                                    icons = [surface_manager.add_icon(
                                                            'axe',
                                                            ':battle-axe.png',
                                                            ')',
                                                            ['dim', 'cyan'])],
                                    types = ['weapon',
                                             'two-handed',
                                             'melee',
                                             'martial weapon'],
                                    rarity = 'uncommon',
                                    weapon_configration = WeaponConfiguration(
                                            damage = [(2, 'crushing'),
                                                      (2, 'slashing')],
                                            critical_range = 12,
                                            critical_damage = 2,
                                            weapon_class = 'martial')))

    config.append(
                  ItemConfiguration(name = 'club',
                                    description = 'Simple and easy to use weapon, that can be used effectively with one hand.',
                                    cost = 6,
                                    weight = 4,
                                    icons = [surface_manager.add_icon(
                                                            'club',
                                                            ':mace.png',
                                                            ')',
                                                            ['dim', 'cyan'])],
                                    types = ['weapon',
                                             'one-handed',
                                             'melee',
                                             'simple weapon'],
                                    rarity = 'common',
                                    weapon_configration = WeaponConfiguration(
                                            damage = [(3, 'crushing')],
                                            critical_range = 11,
                                            critical_damage = 3,
                                            weapon_class = 'simple')))

    config.append(
                  ItemConfiguration(name = 'warhammer',
                                    description = 'Heavy two-handed weapon that crushes through but the strongest defences. Rare sight in the battle field, because of the huge weight.',
                                    cost = 50,
                                    weight = 4,
                                    icons = [surface_manager.add_icon(
                                                            'warhammer',
                                                            ':gavel.png',
                                                            ')',
                                                            ['dim', 'cyan'])],
                                    types = ['weapon',
                                             'two-handed',
                                             'melee',
                                             'martial weapon'],
                                    rarity = 'rare',
                                    weapon_configration = WeaponConfiguration(
                                            damage = [(7, 'crushing')],
                                            critical_range = 12,
                                            critical_damage = 7,
                                            weapon_class = 'martial')))

    config.append(
                  ItemConfiguration(name = 'spear',
                                    description = 'One handed piercing weapon that can be devastating in hands of skilled user.',
                                    cost = 25,
                                    weight = 4,
                                    icons = [surface_manager.add_icon(
                                                            'spear',
                                                            ':barbed-spear.png',
                                                            ')',
                                                            ['dim', 'cyan'])],
                                    types = ['weapon',
                                             'one-handed',
                                             'melee',
                                             'martial weapon'],
                                    rarity = 'rare',
                                    weapon_configration = WeaponConfiguration(
                                            damage = [(5, 'piercing')],
                                            critical_range = 11,
                                            critical_damage = 5,
                                            weapon_class = 'martial')))

    config.append(
                  ItemConfiguration(name = 'whip',
                                    description = 'Weapon of choice for archeologists and vampire hunters. Not particularly strong, but very versatile.',
                                    cost = 25,
                                    weight = 4,
                                    icons = [surface_manager.add_icon(
                                                            'whip',
                                                            ':whip.png',
                                                            ')',
                                                            ['dim', 'yellow'])],
                                    types = ['weapon',
                                             'one-handed',
                                             'melee',
                                             'exotic weapon'],
                                    rarity = 'common',
                                    weapon_configration = WeaponConfiguration(
                                            damage = [(3, 'slashing')],
                                            critical_range = 11,
                                            critical_damage = 3,
                                            weapon_class = 'exotic')))

    config.append(
                  ItemConfiguration(name = 'sickle',
                                    description = 'Farming implement repurposed on the battle field. Light and common weapon that requires very specifically trained user.',
                                    cost = 6,
                                    weight = 4,
                                    icons = [surface_manager.add_icon(
                                                            'sickle',
                                                            ':scythe.png',
                                                            ')',
                                                            ['dim', 'cyan'])],
                                    types = ['weapon',
                                             'light weapon',
                                             'melee',
                                             'exotic weapon'],
                                    rarity = 'common',
                                    weapon_configration = WeaponConfiguration(
                                            damage = [(3, 'slashing')],
                                            critical_range = 10,
                                            critical_damage = 3,
                                            weapon_class = 'exotic')))

    config.append(
                  ItemConfiguration(name = 'morning star',
                                    description = 'Spiked club, guaranteed to pierce through armours and crush through bones. Difficult to use weapon that requires years of practice.',
                                    cost = 40,
                                    weight = 4,
                                    icons = [surface_manager.add_icon(
                                                            'morning star',
                                                            ':spiked-mace.png',
                                                            ')',
                                                            ['dim', 'cyan'])],
                                    types = ['weapon',
                                             'one-handed',
                                             'melee',
                                             'martial weapon'],
                                    rarity = 'uncommon',
                                    weapon_configration = WeaponConfiguration(
                                            damage = [(2, 'crushing'),
                                                      (2, 'piercing')],
                                            critical_range = 12,
                                            critical_damage = 2,
                                            weapon_class = 'martial')))
    config.append(
                  ItemConfiguration(name = 'bow',
                                    description = 'Short bow, built from laminated bones and wood. Perfect for ranged combat.',
                                    cost = 10,
                                    weight = 3,
                                    icons = [surface_manager.add_icon(
                                                            'bow',
                                                            ':bow.png',
                                                            ')',
                                                            ['dim', 'yellow'])],
                                    types = ['weapon',
                                             'two-handed',
                                             'ranged',
                                             'martial weapon'],
                                    rarity = 'uncommon',
                                    weapon_configration = WeaponConfiguration(
                                            damage = [(1, 'crushing')],
                                            critical_range = 12,
                                            critical_damage = 1,
                                            weapon_class = 'martial',
                                            ammunition_type = 'arrow')))
    config.append(
                  ItemConfiguration(name = 'arrow',
                                    description = 'Wooden arrows tipped with metal head.',
                                    cost = 1,
                                    weight = 1,
                                    icons = [surface_manager.add_icon(
                                                            'arrows',
                                                            ':arrows.png',
                                                            ')',
                                                            ['dim', 'yellow'])],
                                    types = ['ammunition'],
                                    rarity = 'uncommon',
                                    ammunition_configuration = AmmunitionConfiguration(
                                            damage = [(3, 'piercing')],
                                            critical_range = 12,
                                            critical_damage = 3,
                                            ammunition_type = 'arrow',
                                            count = 15)))
    config.append(
                  ItemConfiguration(name = 'war arrow',
                                    description = 'Wooden arrows tipped with metal head that have cutting edge.',
                                    cost = 1,
                                    weight = 1,
                                    icons = [surface_manager.add_icon(
                                                            'arrows',
                                                            ':arrows.png',
                                                            ')',
                                                            ['dim', 'yellow'])],
                                    types = ['ammunition'],
                                    rarity = 'uncommon',
                                    ammunition_configuration = AmmunitionConfiguration(
                                            damage = [(1, 'piercing'),
                                                      (2, 'slashing')],
                                            critical_range = 12,
                                            critical_damage = 3,
                                            ammunition_type = 'arrow',
                                            count = 15)))
    config.append(
                  ItemConfiguration(name = 'blunt arrow',
                                    description = 'Wooden arrows with wide end, used to cause blunt trauma.',
                                    cost = 1,
                                    weight = 1,
                                    icons = [surface_manager.add_icon(
                                                            'arrows',
                                                            ':arrows.png',
                                                            ')',
                                                            ['dim', 'yellow'])],
                                    types = ['ammunition'],
                                    rarity = 'uncommon',
                                    ammunition_configuration = AmmunitionConfiguration(
                                            damage = [(3, 'crushing')],
                                            critical_range = 12,
                                            critical_damage = 3,
                                            ammunition_type = 'arrow',
                                            count = 15)))

    config.append(
                  ItemConfiguration(name = 'robes',
                                    description = 'Simple robes are favoured by spell casters. They do not hinder magical abilities and strong wizards are more than capable of deflecting attacks anyway.',
                                    cost = 5,
                                    weight = 4,
                                    icons = [surface_manager.add_icon(
                                                            'robes',
                                                            ':robes.png',
                                                            '[',
                                                            ['dim', 'yellow'])],
                                    types = ['armour'],
                                    rarity = 'common',
                                    armour_configuration = ArmourConfiguration(
                                                damage_reduction = 0,
                                                speed_modifier = 1.0)))
    config.append(
                  ItemConfiguration(name = 'leather armour',
                                    description = 'Armour made of hardened leather. While it offers only meager protection, it allows the wearer to move very swiftly.',
                                    cost = 10,
                                    weight = 6,
                                    icons = [surface_manager.add_icon(
                                                            'leather armour',
                                                            ':leather_armour.png',
                                                            '[',
                                                            ['dim', 'yellow'])],
                                    types = ['armour'],
                                    rarity = 'common',
                                    armour_configuration = ArmourConfiguration(
                                                damage_reduction = 1,
                                                speed_modifier = 1.0)))
    config.append(
                  ItemConfiguration(name = 'chainmail',
                                    description = 'Armour made of countless interlocking metal rings. Somewhat heavier than leather armour.',
                                    cost = 20,
                                    weight = 12,
                                    icons = [surface_manager.add_icon(
                                                            'chainmail',
                                                            ':chainmail.png',
                                                            '[',
                                                            ['dim', 'cyan'])],
                                    types = ['armour'],
                                    rarity = 'uncommon',
                                    armour_configuration = ArmourConfiguration(
                                                damage_reduction = 2,
                                                speed_modifier = 0.9)))
    config.append(
                  ItemConfiguration(name = 'scale mail',
                                    description = 'Scale mail is made of interlocking scales, in a similar manner to chainmail. It offers better protection, while being heavier and harder to move around with.',
                                    cost = 25,
                                    weight = 12,
                                    icons = [surface_manager.add_icon(
                                                            'scale mail',
                                                            ':scale_mail.png',
                                                            '[',
                                                            ['dim', 'cyan'])],
                                    types = ['armour'],
                                    rarity = 'uncommon',
                                    armour_configuration = ArmourConfiguration(
                                                damage_reduction = 3,
                                                speed_modifier = 0.7)))
    config.append(
                  ItemConfiguration(name = 'plate mail',
                                    description = 'Ultimate solution for protection. Heavy plates cover wearer from head to toe, offering great protection against attacks.',
                                    cost = 50,
                                    weight = 4,
                                    icons = [surface_manager.add_icon(
                                                            'plate mail',
                                                            ':plate_mail.png',
                                                            '[',
                                                            ['dim', 'cyan'])],
                                    types = ['armour'],
                                    rarity = 'rare',
                                    armour_configuration = ArmourConfiguration(
                                                damage_reduction = 5,
                                                speed_modifier = 0.5)))
    config.append(
                  ItemConfiguration(name = 'healing potion',
                                    cost = 150,
                                    weight = 1,
                                    icons = [surface_manager.add_icon(
                                                            'plain_potion',
                                                            ':plain_potion.png',
                                                            '!',
                                                            ['red'])],
                                    types = ['potion'],
                                    rarity = 'rare',
                                    effect_handles = [EffectHandle(
                                            trigger = 'on drink',
                                            effect = 'cure medium wounds',
                                            parameters = None,
                                            charges = 1)]))
    config.append(
                  ItemConfiguration(name = 'Tome of Um\'bano - Page 150',
                                    description = 'In crypt under Herculeum, I encountered giant spiders. These arachnids have poisoned bite and could easily take down unprepared adventurer. Dealing with them from distance seems to be the best approach, if cramped surroundings just allow for it.',
                                    cost = 100,
                                    weight = 1,
                                    icons = [surface_manager.add_icon(
                                                            'tied-scroll',
                                                            ':tied-scroll.png',
                                                            '?',
                                                            ['dim', 'yellow'])],
                                    types = ['tome'],
                                    rarity = 'rare'))
    config.append(
                  ItemConfiguration(name = 'Tome of Um\'bano - Page 158',
                                    description = 'Patrolling skeleton warriors have been easy to avoid so far. They patrol mindlessly around the catacombs and only turn to pursue if I approach too close. They lose interest soon after I retreat and return to their endless patrol.',
                                    cost = 100,
                                    weight = 1,
                                    icons = [surface_manager.add_icon(
                                                            'tied-scroll',
                                                            ':tied-scroll.png',
                                                            '?',
                                                            ['dim', 'yellow'])],
                                    types = ['tome'],
                                    rarity = 'rare'))
    config.append(
                  ItemConfiguration(name = 'Tome of Um\'bano - Page 326',
                                    description = 'In the ancient times, old evil ruled over the land and terrorized everything. Kingdoms were drowned in fire and destroyed by demon, Crimson Jaw. Only when seven sages combined their powers, they could overcome him. But Crimson Jaw was too strong to be destroyed completely and his spirit had to be bound and trapped. Crimson Lair is his final resting place, where seals of seven sages keep him imprisoned.',
                                    cost = 100,
                                    weight = 1,
                                    icons = [surface_manager.add_icon(
                                                            'tied-scroll',
                                                            ':tied-scroll.png',
                                                            '?',
                                                            ['dim', 'yellow'])],
                                    types = ['tome'],
                                    rarity = 'rare'))
    config.append(
                  ItemConfiguration(name = 'Tome of Um\'bano - Page 485',
                                    description = 'The whip of Ashmque is a horrendous weapon. Thong of the whip is made of blackened links of steel and an eternal flame is burning around it. Only the strongest of men can even try to wield it because of the heavy construction. It is said that the whip of Ashmque was forged at the dawn of ages by demon smith Ashmque.',
                                    cost = 100,
                                    weight = 1,
                                    icons = [surface_manager.add_icon(
                                                            'tied-scroll',
                                                            ':tied-scroll.png',
                                                            '?',
                                                            ['dim', 'yellow'])],
                                    types = ['tome'],
                                    rarity = 'rare'))
    config.append(
                  ItemConfiguration(name = 'Tome of Um\'bano - Page 612',
                                    description = 'Prince Razel was obsessed with alchemy and built a complex laboratory under city of Herculeum. Untold experiments were conducted there and it is rumoured that those experiments ultimately caused his death.',
                                    cost = 100,
                                    weight = 1,
                                    icons = [surface_manager.add_icon(
                                                            'tied-scroll',
                                                            ':tied-scroll.png',
                                                            '?',
                                                            ['dim', 'yellow'])],
                                    types = ['tome'],
                                    rarity = 'rare'))
    config.append(
                  ItemConfiguration(name = 'Tome of Um\'bano - Page 615',
                                    description = 'Tomb of Prince Razel is rumoured to have been filled with traps and experiments he conducted while he was still living. The exact location of the tomb has been lost in the mists of time, but I think it has to be near his hidden laboratory.',
                                    cost = 100,
                                    weight = 1,
                                    icons = [surface_manager.add_icon(
                                                            'tied-scroll',
                                                            ':tied-scroll.png',
                                                            '?',
                                                            ['dim', 'yellow'])],
                                    types = ['tome'],
                                    rarity = 'rare'))
    config.append(
                  ItemConfiguration(name = 'Tome of Um\'bano - Page 621',
                                    description = 'Prince Razel\'s greatest experiment was a staff that could shoot fire on command. He never revealed how he created it and guarded the staff jealously.',
                                    cost = 100,
                                    weight = 1,
                                    icons = [surface_manager.add_icon(
                                                            'tied-scroll',
                                                            ':tied-scroll.png',
                                                            '?',
                                                            ['dim', 'yellow'])],
                                    types = ['tome'],
                                    rarity = 'rare'))
    return config



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

#   Copyright 2010-2013 Tuukka Turto
#
#   This file is part of pyherc.
#
#   pyherc is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#
#   pyherc is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with pyherc.  If not, see <http://www.gnu.org/licenses/>.

"""
Module containing classes to represent Level
"""

import random
from pyherc.aspects import logged

class Level():
    """
    Represents a level
    """
    @logged
    def __init__(self, size = (0, 0), floor_type = None, wall_type = None,
                 empty_floor = 0, empty_wall = 0):
        """
        Initialises a level of certain size and fill floor and walls with given types

        :param size: optional size of the level
        :type size: (integer, integer)
        :param floor_type: type of floor to fill level
        :type floor_type: integer
        :param wall_type: type of wall to fill level
        :type wall_type: integer
        :empty_floor: type of floor for empty
        :type empty_floor: integer
        :empty_wall: type of wall for empty
        :type empty_wall: integer
        """
        super(Level, self).__init__()

        self.floor = []
        self.walls = []
        self.empty_floor = empty_floor
        self.empty_wall = empty_wall
        self.__location_type = []
        self.lit = []

        if size[0] != 0 and size[1] != 0:
            for loc_x in range(0, size[0] + 1):
                temp_row = []
                for loc_y in range(0, size[1] + 1):
                    temp_row.append(floor_type)
                self.floor.append(temp_row)

            for loc_x in range(0, size[0] + 1):
                temp_row = []
                for loc_y in range(0, size[1] + 1):
                    temp_row.append(wall_type)
                self.walls.append(temp_row)

            for loc_x in range(0, size[0] + 1):
                temp_row = []
                for loc_y in range(0, size[1] + 1):
                    temp_row.append(None)
                self.__location_type.append(temp_row)

        self.items = []
        self.portals = []
        self.creatures = []
        self.full_update_needed = True
        self.dirty_rectangles = []

    def get_tile(self, loc_x, loc_y):
        """
        Get tile at given location

        :param loc_x: x-coordinate of the location
        :type loc_x: integer
        :param loc_y: y-coordinate of the location
        :type loc_y: integer
        :returns: tile ID at given location
        :rtype: int
        """
        if loc_x < 0 or loc_y < 0:
            return self.empty_floor

        if loc_x > len(self.floor) or loc_y > len(self.floor[0]):
            return self.empty_floor

        if self.walls[loc_x][loc_y] != self.empty_wall:
            return self.walls[loc_x][loc_y]
        else:
            return self.floor[loc_x][loc_y]

    def get_wall_tile(self, loc_x, loc_y):
        """
        Get wall tile at given location

        :param loc_x: x-coordinate of the location
        :type loc_x: integer
        :param loc_y: y-coordinate of the location
        :type loc_y: integer
        :returns: wall tile ID at given location
        :rtype: int
        """
        if loc_x < 0 or loc_y < 0:
            return self.empty_wall #?

        if loc_x > len(self.floor) or loc_y > len(self.floor[0]):
            return self.empty_wall

        return self.walls[loc_x][loc_y]

    @logged
    def add_item(self, item, location):
        """
        Add an item to this level

        :param item: item to add
        :type item: Item
        :param location: location where to put the item
        :type location: (integer, integer)
        """
        assert(not item == None)
        assert(not location == None)

        self.items.append(item)
        item.location = location
        item.level = self

    def get_items_at(self, location):
        """
        Get list of items at location

        :param location: location to check
        :type location: (integer, integer)
        :returns: items at given location
        :rtype: [Item]
        """
        assert(location != None)
        items = []
        for item in self.items:
            if item.location == location:
                items.append(item)
        return items

    @logged
    def add_portal(self, portal, location, other_end = None):
        """
        Adds precreated portal on level at given location
        If secondary portal is specified, link them together

        :param portal: portal to add
        :type portal: Portal
        :param location: location where to add portal
        :type location: (integer, integer)
        :param other_end: optional other end of the portal
        :type other_end: Portal
        """
        assert(portal != None)
        assert(location != None)

        portal.level = self
        portal.location = location
        self.portals.append(portal)

        if other_end != None:
            assert(other_end.icon != None or portal.icon != None)

            portal.set_other_end(other_end)
            other_end.set_other_end(portal)

            #TODO: remove link
            if portal.icon != None:
                if other_end.icon == None:
                    if portal.icon == 201:
                        other_end.icon = 200
                    else:
                        other_end.icon = 201
            else:
                if other_end.icon == 201:
                    portal.icon = 200
                else:
                    portal.icon = 201

    def get_portal_at(self, location):
        """
        Check if there is a portal at given location

        :param location: location of portal
        :type location: (integer, integer)
        :returns: Portal if found, otherwise None
        :rtype: Portal
        """
        for portal in self.portals:
            if portal.location == location:
                return portal

        return None

    @logged
    def add_creature(self, creature, location = None):
        """
        Add a creature to level

        :param creature: creature to add
        :type creature: Creature
        :param location: optional location for the creature
        :type location: (integer, integer)
        """
        assert(creature != None)

        self.creatures.append(creature)
        creature.level = self
        if location != None:
            creature.location = location

    @logged
    def remove_creature(self, creature):
        """
        Remove creature from level

        :param creature: creature to remove
        :type creature: Creature
        """
        assert(creature != None)
        assert(creature in self.creatures)

        self.creatures.remove(creature)
        creature.level = None
        creature.location = ()

    def get_creature_at(self, location):
        """
        Get list of creatures at given location

        :param location: location to check
        :type location: (integer, integer)
        :returns: creature if found
        :rtype: Creature
        """
        assert(location != None)
        for creature in self.creatures:
            if creature.location == location:
                return creature

        return None

    @logged
    def find_free_space(self):
        """
        Finds free space where stuff can be placed

        :returns: Location where space is free
        :rtype: (int, int)
        """
        width = len(self.floor)
        height = len(self.floor[0])
        location = (random.randint(2, width - 1), random.randint(2, height - 1))
        while self.walls[location[0]][location[1]] != self.empty_wall:
            location = (random.randint(2, width - 1), random.randint(2, height - 1))
        return location

    def get_square(self, x_coordinate, y_coordinate):
        """
        Get square at given coordinates

        :param x_coordinate: x-coorinate of location
        :type x_coordinate: integer
        :param y_coordinate: y-coordinate of location
        :type y_coordinate: integer
        :returns: icon ID of the tile
        :rtype: int
        """
        if self.walls[x_coordinate][y_coordinate] != self.empty_wall:
            return self.walls[x_coordinate][y_coordinate]
        else:
            return self.floor[x_coordinate][y_coordinate]

    def blocks_movement(self, loc_x, loc_y):
        """
        Checks if there's movement blocking wall at given coordinates

        :param loc_x: x-coordinate of the location
        :type loc_x: integer
        :param loc_Y: y-coordinate of the location
        :type loc_y: integer
        :returns: True if location blocks regular movement, otherwise False
        :rtype: Boolean
        """
        if loc_x < 0 or loc_y < 0:
            return True

        if loc_x >= len(self.walls) or loc_y >= len(self.walls[0]):
            return True

        if self.walls[loc_x][loc_y] == self.empty_wall:
            return False
        else:
            return True

    def blocks_los(self, x_coordinate, y_coordinate):
        """
        Checks if there's LOS-blocking wall at given coordinates

        :param x_coordinate: x-coordinate of the location
        :type x_coordinate: integer
        :param y_coordinate: y-coordinate of the location
        :type y_coordinate: integer
        :returns: True if location blocks line of sight, otherwise False
        :rtype: Boolean
        """

        if x_coordinate < 0 or y_coordinate < 0:
            return True

        if x_coordinate >= len(self.walls) or y_coordinate >= len(self.walls[0]):
            return True

        if self.walls[x_coordinate][y_coordinate] == self.empty_wall:
            return False
        else:
            return True

    @logged
    def get_size(self):
        """
        Gets size of level

        :returns: tupple, with width and length of level
        :rtype: (int, int)
        """
        x_size = len(self.floor)
        y_size = len(self.floor[0])

        return (x_size, y_size)

    @logged
    def set_location_type(self, location, location_type):
        """
        Set type of location

        :param  location: location to set
        :type location: (integer, integer)
        :param location_type: type of location
        :type location_type: int
        """
        self.__location_type[location[0]][location[1]] = location_type

    @logged
    def get_location_type(self, location):
        """
        Get type of location

        :param location: location to get
        :type location: (integer, integer)
        :returns: Type of location
        :rtype: int
        """
        return self.__location_type[location[0]][location[1]]

    @logged
    def get_locations_by_type(self, location_type):
        """
        Get locations marked as rooms

        :param location_type: Type of location to search.
        :type location_type: string
        :returns: locations identifying rooms
        :rtype: [(int, int)]
        """
        rooms = []
        for loc_x in range(len(self.__location_type)):
            for loc_y in range(len(self.__location_type[0])):
                if (self.__location_type[loc_x][loc_y] == location_type or
                        self.__location_type[loc_x][loc_y] != None
                        and location_type == 'any'):
                    rooms.append((loc_x, loc_y))

        return rooms

    def dump_string(self):
        """
        Dump this level into a string

        :returns: ascii representation of level
        :rtype: list of list
        """
        level_string = ""
        size = self.get_size()
        for loc_y in range(size[1]):
            level_string = level_string + '\n'
            for loc_x in range(size[0]):
                creature = self.get_creature_at((loc_x, loc_y))
                portal = self.get_portal_at((loc_x, loc_y))
                items = self.get_items_at((loc_x, loc_y))

                if creature != None:
                    level_string = level_string + "X"
                elif portal != None:
                    level_string = level_string + "<"
                elif len(items) > 0:
                    level_string = level_string + "*"
                elif self.walls[loc_x][loc_y] != self.empty_wall:
                    level_string = level_string + "#"
                elif self.walls[loc_x][loc_y] != self.empty_floor:
                    level_string = level_string + "."
                else:
                    level_string = level_string + " "
        return level_string

    def _repr_pretty_(self, p, cycle):
        """
        Pretty print for IPython
        """
        level_string = self.dump_string()

        if cycle:
            p.text('Level(...)')
        else:
            for char in level_string:
                if char != '\n':
                    p.text(char)
                else:
                    p.breakable()

    def heuristic_estimate_of_distance(self, start, goal):
        """
        This should be >= 0
        If you want to be sure, that the found path is the sortest one,
        let this return a constant 0.
        """
        l = len(goal)
        return (sum([(start[i] - goal[i]) ** 2 for i in range(l)])) ** 0.5

    def neighbor_nodes(self, node):
        """
        node : the node of which neighbours you want to get
        It might also be a good choise to implement a class for the node
        """
        nodes = []
        loc_x = node[0]
        loc_y = node[1]
        for x in range(loc_x - 1, loc_x + 2):
            for y in range(loc_y -1, loc_y + 2):
                if (x != loc_x or y != loc_y) and not self.blocks_movement(x, y):
                    nodes.append((x, y))

        return nodes

    def dist_between(self, start, goal):
        """
        The real distance between two adjacent nodes.
        This should be >= 0
        """
        l = len(goal)
        return (sum([(start[i] - goal[i]) ** 2 for i in range(l)])) ** 0.5



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

#   Copyright 2010-2013 Tuukka Turto
#
#   This file is part of pyherc.
#
#   pyherc is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#
#   pyherc is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with pyherc.  If not, see <http://www.gnu.org/licenses/>.

"""
Module for application level objects
"""
from herculeum.config import Configuration
from pyherc.data.model import Model
import sys
import logging
import herculeum.config.levels
import hy

class Application():
    """
    This class represents main application
    """

    def __init__(self):
        super(Application, self).__init__()
        self.config = None
        self.gui = None
        self.world = None
        self.running = 1
        self.logger = None
        self.screen = None
        self.log_level = None
        self.ui_mode = 'qt'

    def process_command_line(self):
        """
        Process command line options
        """
        log_levels = {'debug': logging.DEBUG,
                      'info': logging.INFO,
                      'warning': logging.WARNING,
                      'error': logging.ERROR,
                      'critical': logging.CRITICAL}
        args = sys.argv
        for argument in args:
            if argument in log_levels:
                self.log_level = log_levels[argument]

        modes = ['qt', 'curses']
        for argument in args:
            if argument in modes:
                self.ui_mode = argument

    def load_configuration(self, controls, surface_manager):
        """
        Load configuration
        """
        self.world = Model()
        self.config = Configuration(self.world,
                                    herculeum.config.levels,
                                    controls,
                                    surface_manager)

        self.config.initialise()

    def run(self, user_interface):
        """
        Starts the application
        """
        user_interface.show_main_window()

    def enable_cheat(self):
        """
        Enables cheat mode

        .. versionadded:: 0.9
        """
        print('Cheat code activated')
        for spec in self.level_generator_factory.portal_adder_configurations:
            spec.chance = 100

    def __get_surface_manager(self):
        """
        Get surface manager
        """
        return self.config.surface_manager

    def start_logging(self):
        """
        Start logging for the system
        """
        logging.basicConfig(filename='pyherc.log',
                            level=self.log_level)
        self.logger = logging.getLogger('pyherc.main.Application')
        self.logger.info("Logging started")

    def __get_action_factory(self):
        """
        Get action factory instance

        Returns:
            ActionFactory
        """
        return self.config.action_factory

    def __get_creature_generator(self):
        """
        Get creature generator

        Returns:
            CreatureGenerator
        """
        return self.config.creature_generator

    def __get_item_generator(self):
        """
        Get item generator

        Returns:
            ItemGenerator
        """
        return self.config.item_generator

    def __get_level_generator_factory(self):
        """
        Get level generator factory
        """
        return self.config.level_generator_factory

    def __get_rng(self):
        """
        Get random number generator
        """
        return self.config.rng

    def __get_rules_engine(self):
        """
        Get rules engine
        """
        return self.config.rules_engine

    def __get_player_generator(self):
        """
        Get player generator
        """
        return self.config.player_generator

    surface_manager = property(__get_surface_manager)
    action_factory = property(__get_action_factory)
    creature_generator = property(__get_creature_generator)
    player_generator = property(__get_player_generator)
    item_generator = property(__get_item_generator)
    level_generator_factory = property(__get_level_generator_factory)
    rng = property(__get_rng)
    rules_engine = property(__get_rules_engine)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

#   Copyright 2010-2013 Tuukka Turto
#
#   This file is part of pyherc.
#
#   pyherc is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#
#   pyherc is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with pyherc.  If not, see <http://www.gnu.org/licenses/>.

"""
Package for generation related activities
"""

from .dungeon import DungeonGenerator
from .item import ItemGenerator, ItemConfigurations, ItemConfiguration
from .item import WeaponConfiguration, ArmourConfiguration
from .item import AmmunitionConfiguration
from .creature import CreatureGenerator
from .creature import CreatureConfiguration
from .creature import InventoryConfiguration
from .effects import EffectsFactory
from .spells import SpellGenerator



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

#   Copyright 2010-2013 Tuukka Turto
#
#   This file is part of pyherc.
#
#   pyherc is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#
#   pyherc is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with pyherc.  If not, see <http://www.gnu.org/licenses/>.

"""
Module for effects collection
"""
import collections

class EffectsCollection():
    """
    Class for representing collection of effects

    .. versionadded:: 0.4
    """
    def __init__(self):
        """
        Default constructor
        """
        super(EffectsCollection, self).__init__()
        self.handles = {}
        self.effects = []

    def add_effect_handle(self, handle):
        """
        Add effect handle

        :param handle: effect handle to add
        :type handle: EffectHandle
        """
        assert handle != None

        handles = self.handles
        trigger = handle.trigger

        if not trigger in handles:
            handles[trigger] = []
        handles[trigger].append(handle)

    def get_effect_handles(self, trigger = None):
        """
        Get effect handles

        :param trigger: optional trigger type
        :type trigger: string

        :returns: effect handles
        :rtype: [EffectHandle]
        """
        if trigger is None:
            effects = []
            for key in self.handles:
                for handle in self.handles[key]:
                    effects.append(handle)
        else:
            if trigger in self.handles:
                effects = [x for x in self.handles[trigger]]
            else:
                effects = []

        return effects

    def remove_effect_handle(self, handle):
        """
        Remove given handle

        :param handle: handle to remove
        :type handle: EffectHandle
        """
        assert handle != None

        for key, value in self.handles.items():
            if handle in value:
                value.remove(handle)

    def has_effect(self, effect):
        """
        Check if given type of effect exists in collection

        :param effect: effect to check
        :type effect: Effect
        """
        return effect.effect_name in [x.effect_name for x in self.effects]

    def add_effect(self, effect):
        """
        Add effect

        :param effect: effect to add
        :type effect: Effect
        """
        assert effect != None

        self.effects.append(effect)

    def get_effects(self):
        """
        Get effects from collection

        :returns: effects
        :rtype: [Effect]
        """
        return [x for x in self.effects]

    def get_expired_effects(self):
        """
        Get expired effects

        :returns: expired effects
        :rtype: [Effect]
        """
        return [x for x in self.effects
                        if x.duration != None
                        and x.duration <= 0]

    def remove_expired_effects(self):
        """
        Remove expired effects from collection
        """
        self.effects = [x for x in self.effects
                                if x.duration > 0]

    def get_charges_left(self):
        """
        Amount of charges left in collection

        :returns: amount of charges
        :rtype: [integer]
        """
        if len(self.get_effect_handles()) == 0:
            return []

        return [x.charges for x in self.get_effect_handles()]

    def get_maximum_charges_left(self):
        """
        Return highest amount of charges left in collection

        :returns: highest charge
        :rtype: integer
        """
        charges = self.get_charges_left()

        if charges != None:
            if isinstance(charges, collections.Sequence):
                if len(charges) > 0:
                    return max(charges)
                else:
                    return None
            else:
                return charges
        else:
            return None

    def get_minimum_charges_left(self):
        """
        Return smallest amount of charges left in item

        :returns: smallest charge in collection
        :rtype: integer
        """
        charges = self.get_charges_left()

        if charges != None:
            return min(charges)
        else:
            return None

    def _repr_pretty_(self, p, cycle):
        """
        Pretty print for IPython

        :param p: printer to write
        :param cycle: has pretty print detected a cycle?
        """
        if cycle:
            p.text('EffectsCollection(...)')
        else:
            p.text('Effects:')
            for effect in self.effects:
                p.pretty(effect)
                p.breakable()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

#   Copyright 2010-2013 Tuukka Turto
#
#   This file is part of pyherc.
#
#   pyherc is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#
#   pyherc is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with pyherc.  If not, see <http://www.gnu.org/licenses/>.

"""
Package for events that are used to communicate between creatures and UI
"""
from .event import Event
from .move import MoveEvent
from .combat import AttackHitEvent, AttackNothingEvent, AttackMissEvent
from .poison import PoisonTriggeredEvent, PoisonAddedEvent, PoisonEndedEvent
from .healing import HealTriggeredEvent, HealAddedEvent, HealEndedEvent
from .death import DeathEvent
from .inventory import PickUpEvent, DropEvent, EquipEvent, UnEquipEvent
from .hitpoints import HitPointsChangedEvent
from .damage import DamageAddedEvent, DamageTriggeredEvent, DamageEndedEvent
from .perception import LoseFocusEvent, NoticeEvent
from .error import ErrorEvent



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

#   Copyright 2010-2013 Tuukka Turto
#
#   This file is part of pyherc.
#
#   pyherc is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#
#   pyherc is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with pyherc.  If not, see <http://www.gnu.org/licenses/>.

"""
Attack related factories are defined here
"""
from pyherc.aspects import logged
from pyherc.rules.attack.action import AttackAction, AdditionalRules
from pyherc.rules.attack.unarmed import UnarmedToHit
from pyherc.rules.attack.unarmed import UnarmedDamage
from pyherc.rules.attack.melee import MeleeToHit
from pyherc.rules.attack.melee import MeleeDamage
from pyherc.rules.factory import SubActionFactory

class AttackFactory(SubActionFactory):
    """
    Factory for constructing attack actions
    """

    @logged
    def __init__(self, factories):
        """
        Constructor for this factory
        """
        super(AttackFactory, self).__init__()
        self.action_type = 'attack'

        if hasattr(factories, '__iter__'):
            self.factories = factories
        else:
            self.factories = []
            self.factories.append(factories)


class UnarmedCombatFactory():
    """
    Factory for producing unarmed combat actions
    """
    @logged
    def __init__(self, effect_factory, dying_rules):
        """
        Constructor for this factory
        """
        self.attack_type = 'unarmed'
        self.effect_factory = effect_factory
        self.dying_rules = dying_rules

    def __str__(self):
        return 'unarmed combat factory'

    @logged
    def can_handle(self, parameters):
        """
        Can this factory process these parameters

        :param parameters: parameters to check
        :type parameters: UnarmedAttackParameters
        :returns: true if factory is capable of handling parameters
        :rtype: boolean
        """
        return self.attack_type == parameters.attack_type

    @logged
    def get_action(self, parameters):
        """
        Create a attack action

        :param parameters: parameters used to control attack creation
        :type parameters: UnarmedAttackParameters
        :returns: action that can be executed
        :rtype: AttackAction
        """
        attacker = parameters.attacker
        target = self.get_target(parameters)
        damage = [(attacker.get_attack(), 'crushing')]

        attack = AttackAction('unarmed',
                    to_hit = UnarmedToHit(attacker, target,
                                          parameters.random_number_generator),
                    damage = UnarmedDamage(damage = damage),
                    attacker = attacker,
                    target = target,
                    effect_factory = self.effect_factory,
                    dying_rules = self.dying_rules,
                    additional_rules = AdditionalRules(attacker))

        return attack

    @logged
    def get_target(self, parameters):
        """
        Get target of the attack

        :param parameters: attack parameters
        :type parameters: UnarmedAttackParameters
        :returns: target character if found, otherwise None
        :rtype: Character
        """
        attacker = parameters.attacker
        location = attacker.location
        direction = parameters.direction
        level = attacker.level

        if direction == 1:
            target_location = (location[0], location[1] - 1)
        elif direction == 2:
            target_location = (location[0] + 1, location[1] - 1)
        elif direction == 3:
            target_location = (location[0] + 1, location[1])
        elif direction == 4:
            target_location = (location[0] + 1, location[1] + 1)
        elif direction == 5:
            target_location = (location[0], location[1] + 1)
        elif direction == 6:
            target_location = (location[0] - 1, location[1] + 1)
        elif direction == 7:
            target_location = (location[0] - 1, location[1])
        elif direction == 8:
            target_location = (location[0] - 1, location[1] - 1)
        else:
            target_location = None

        if target_location != None:
            target = level.get_creature_at(target_location)
            return target
        else:
            return None

class MeleeCombatFactory():
    """
    Factory for producing melee combat actions
    """
    @logged
    def __init__(self, effect_factory, dying_rules):
        """
        Constructor for this factory
        """
        self.attack_type = 'melee'
        self.effect_factory = effect_factory
        self.dying_rules = dying_rules

    def __str__(self):
        return 'melee combat factory'

    @logged
    def can_handle(self, parameters):
        """
        Can this factory process these parameters

        :param parameters: parameters to check
        :type parameters: MeleeAttackParameters
        :returns: true if factory is capable of handling parameters
        :rtype: boolean
        """
        return self.attack_type == parameters.attack_type

    @logged
    def get_action(self, parameters):
        """
        Create a attack action

        :param parameters: parameters used to control attack creation
        :type parameters: MeleeAttackParameters
        :returns: action that can be executed
        :rtype: AttackAction
        """
        attacker = parameters.attacker
        target = self.get_target(parameters)
        weapon = attacker.inventory.weapon
        weapon_data = weapon.weapon_data

        attack = AttackAction(
                    attack_type = 'melee',
                    to_hit = MeleeToHit(attacker, target,
                                        parameters.random_number_generator),
                    damage = MeleeDamage(damage = weapon_data.damage),
                    attacker = attacker,
                    target = target,
                    effect_factory = self.effect_factory,
                    dying_rules = self.dying_rules,
                    additional_rules = AdditionalRules(attacker))

        return attack

    @logged
    def get_target(self, parameters):
        """
        Get target of the attack

        :param parameters: parameters to control attack
        :type parameters: MeleeAttackParameters
        :returns: target character if found, otherwise None
        :rtype: Character
        """
        attacker = parameters.attacker
        location = attacker.location
        direction = parameters.direction
        level = attacker.level

        if direction == 1:
            target_location = (location[0], location[1] - 1)
        elif direction == 2:
            target_location = (location[0] + 1, location[1] - 1)
        elif direction == 3:
            target_location = (location[0] + 1, location[1])
        elif direction == 4:
            target_location = (location[0] + 1, location[1] + 1)
        elif direction == 5:
            target_location = (location[0], location[1] + 1)
        elif direction == 6:
            target_location = (location[0] - 1, location[1] + 1)
        elif direction == 7:
            target_location = (location[0] - 1, location[1])
        elif direction == 8:
            target_location = (location[0] - 1, location[1] - 1)
        else:
            target_location = None

        target = level.get_creature_at(target_location)

        if target_location != None:
            target = level.get_creature_at(target_location)
            return target
        else:
            return None



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

#   Copyright 2010-2013 Tuukka Turto
#
#   This file is part of pyherc.
#
#   pyherc is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#
#   pyherc is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with pyherc.  If not, see <http://www.gnu.org/licenses/>.

"""
Module defining classes related to Move
"""
from pyherc.events import MoveEvent
from pyherc.aspects import logged
from pyherc.data.model import ESCAPED_DUNGEON

class MoveAction():
    """
    Action for moving
    """
    @logged
    def __init__(self, character, new_location, new_level = None):
        """
        Default constructor

        :param character: character moving
        :type character: Character
        :param new_location: location to move
        :type new_location: (int, int)
        :param new_level: level to move
        :type new_level: Level
        """
        self.character = character
        self.new_location = new_location
        self.new_level = new_level
        self.model = None

    @logged
    def execute(self):
        """
        Executes this Move
        """
        if self.is_legal():

            affected_tiles = [self.character.location,
                              self.new_location]

            self.character.location = self.new_location
            if self.new_level != None:
                if self.character.level != self.new_level:
                    self.character.level.remove_creature(self.character)
                    self.character.level = self.new_level
                    self.new_level.add_creature(self.character,
                                                self.new_location)
            self.character.add_to_tick(2)

            self.character.raise_event(MoveEvent(
                                            mover = self.character,
                                            affected_tiles = affected_tiles))

        else:
            self.character.add_to_tick(1)

    @logged
    def is_legal(self):
        """
        Check if the move is possible to perform

        :returns: True if move is possible, false otherwise
        :rtype: Boolean
        """
        location_ok = False
        if self.new_level != None:
            if not self.new_level.blocks_movement(self.new_location[0],
                                                  self.new_location[1]):
                #check for other creatures and such
                location_ok = True
                creatures = self.new_level.creatures
                for creature in creatures:
                    if creature.location == self.new_location:
                        location_ok = False
            else:
                location_ok = False
        else:
            pass

        return location_ok

class WalkAction(MoveAction):
    """
    Action for walking
    """
    def execute(self):
        """
        Execute this move
        """
        MoveAction.execute(self)

class EscapeAction(MoveAction):
    """
    Action for escaping the dungeon

    .. versionadded:: 0.8
    """
    def __init__(self, character):
        """
        Default constructor
        """
        super(EscapeAction, self).__init__(character = character,
                                           new_location = None,
                                           new_level = None)

    def execute(self):
        """
        Execute this move
        """
        model = self.character.model
        model.end_condition = ESCAPED_DUNGEON

    @logged
    def is_legal(self):
        """
        Check if the move is possible to perform

        :returns: True if move is possible, false otherwise
        :rtype: Boolean
        """
        model = self.character.model

        if model.player == self.character:
            return True
        else:
            return False
